# -*- coding: utf-8 -*-
#!/usr/bin/python
import sys
import os
import cv2 as cv
import numpy as np

def main():
    print('\nDeeptextdetection.py')
    print('       A demo script of text box alogorithm of the paper:')
    print('       * Minghui Liao et al.: TextBoxes: A Fast Text Detector with a Single Deep Neural Network https://arxiv.org/abs/1611.06779\n')

    if (len(sys.argv) < 2):
        print(' (ERROR) You must call this script with an argument (path_to_image_to_be_processed)\n')
        quit()

    if not os.path.isfile('TextBoxes_icdar13.caffemodel') or not os.path.isfile('textbox.prototxt'):
        print " Model files not found in current directory. Aborting"
        print " See the documentation of text::TextDetectorCNN class to get download links."
        quit()

    img = cv.imread(str(sys.argv[1]))
    textSpotter = cv.text.TextDetectorCNN_create("textbox.prototxt", "TextBoxes_icdar13.caffemodel")
    rects, outProbs = textSpotter.detect(img);
    vis = img.copy()
    thres = 0.6

    for r in range(np.shape(rects)[0]):
        if outProbs[r] > thres:
            rect = rects[r]
            cv.rectangle(vis, (rect[0],rect[1]), (rect[0] + rect[2], rect[1] + rect[3]), (255, 0, 0), 2)

    cv.imshow("Text detection result", vis)
    cv.waitKey()

if __name__ == "__main__":
    main()




'''
Showcases the use of background subtraction from a live video feed,
aswell as pass through of a known foreground parameter
'''

# Python 2/3 compatibility
from __future__ import print_function

import numpy as np
import cv2 as cv

def main():
    cap = cv.VideoCapture(0)
    if not cap.isOpened:
        print("Capture source avaialable.")
        exit()

    # Create background subtractor
    mog2_bg_subtractor = cv.createBackgroundSubtractorMOG2(history=300, varThreshold=50, detectShadows=False)
    knn_bg_subtractor = cv.createBackgroundSubtractorKNN(history=300, detectShadows=False)

    frame_count = 0
    # Allows for a frame buffer for the mask to learn pre known foreground
    show_count = 10

    while True:
        ret, frame = cap.read()
        if not ret:
            break

        x = 100 + (frame_count % 10) * 3

        frame = cv.resize(frame, (640, 480))
        aKnownForegroundMask = np.zeros(frame.shape[:2], dtype=np.uint8)

        # Allow for models to "settle"/learn
        if frame_count > show_count:
            cv.rectangle(aKnownForegroundMask, (x,200), (x+50,300), 255, -1)
            cv.rectangle(aKnownForegroundMask, (540,180), (640,480), 255, -1)

        #MOG2 Subtraction
        mog2_with_mask = mog2_bg_subtractor.apply(frame,knownForegroundMask=aKnownForegroundMask)
        mog2_without_mask = mog2_bg_subtractor.apply(frame)

        #KNN Subtraction
        knn_with_mask = knn_bg_subtractor.apply(frame,knownForegroundMask=aKnownForegroundMask)
        knn_without_mask = knn_bg_subtractor.apply(frame)

        # Display the 3 parameter apply and the 4 parameter apply for both subtractors
        cv.imshow("MOG2 With a Foreground Mask", mog2_with_mask)
        cv.imshow("MOG2 Without a Foreground Mask", mog2_without_mask)
        cv.imshow("KNN With a Foreground Mask", knn_with_mask)
        cv.imshow("KNN Without a Foreground Mask", knn_without_mask)

        key = cv.waitKey(30)
        if key == 27:  # ESC
            break

        frame_count += 1

    cap.release()
    cv.destroyAllWindows()

if __name__ == '__main__':
    print(__doc__)
    main()
    cv.destroyAllWindows()



__all__: list[str] = []

import cv2
import cv2.ml
import cv2.typing
import typing as _typing


# Classes
class QualityBase(cv2.Algorithm):
    # Functions
    @_typing.overload
    def compute(self, img: cv2.typing.MatLike) -> tuple[float, float, float, float]: ...
    @_typing.overload
    def compute(self, img: cv2.UMat) -> tuple[float, float, float, float]: ...

    @_typing.overload
    def getQualityMap(self, dst: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
    @_typing.overload
    def getQualityMap(self, dst: cv2.UMat | None = ...) -> cv2.UMat: ...

    def clear(self) -> None: ...

    def empty(self) -> bool: ...


class QualityBRISQUE(QualityBase):
    # Functions
    @_typing.overload
    def compute(self, img: cv2.typing.MatLike) -> tuple[float, float, float, float]: ...
    @_typing.overload
    def compute(self, img: cv2.UMat) -> tuple[float, float, float, float]: ...
    @_typing.overload
    def compute(self, img: cv2.typing.MatLike, model_file_path: str, range_file_path: str) -> tuple[float, float, float, float]: ...
    @_typing.overload
    def compute(self, img: cv2.UMat, model_file_path: str, range_file_path: str) -> tuple[float, float, float, float]: ...

    @classmethod
    @_typing.overload
    def create(cls, model_file_path: str, range_file_path: str) -> QualityBRISQUE: ...
    @classmethod
    @_typing.overload
    def create(cls, model: cv2.ml.SVM, range: cv2.typing.MatLike) -> QualityBRISQUE: ...

    @staticmethod
    @_typing.overload
    def computeFeatures(img: cv2.typing.MatLike, features: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
    @staticmethod
    @_typing.overload
    def computeFeatures(img: cv2.UMat, features: cv2.UMat | None = ...) -> cv2.UMat: ...


class QualityGMSD(QualityBase):
    # Functions
    @_typing.overload
    def compute(self, cmp: cv2.typing.MatLike) -> tuple[float, float, float, float]: ...
    @_typing.overload
    def compute(self, cmp: cv2.UMat) -> tuple[float, float, float, float]: ...
    @_typing.overload
    def compute(self, ref: cv2.typing.MatLike, cmp: cv2.typing.MatLike, qualityMap: cv2.typing.MatLike | None = ...) -> tuple[cv2.typing.Scalar, cv2.typing.MatLike]: ...
    @_typing.overload
    def compute(self, ref: cv2.UMat, cmp: cv2.UMat, qualityMap: cv2.UMat | None = ...) -> tuple[cv2.typing.Scalar, cv2.UMat]: ...

    def empty(self) -> bool: ...

    def clear(self) -> None: ...

    @classmethod
    @_typing.overload
    def create(cls, ref: cv2.typing.MatLike) -> QualityGMSD: ...
    @classmethod
    @_typing.overload
    def create(cls, ref: cv2.UMat) -> QualityGMSD: ...


class QualityMSE(QualityBase):
    # Functions
    @_typing.overload
    def compute(self, cmpImgs: _typing.Sequence[cv2.typing.MatLike]) -> tuple[float, float, float, float]: ...
    @_typing.overload
    def compute(self, cmpImgs: _typing.Sequence[cv2.UMat]) -> tuple[float, float, float, float]: ...
    @_typing.overload
    def compute(self, ref: cv2.typing.MatLike, cmp: cv2.typing.MatLike, qualityMap: cv2.typing.MatLike | None = ...) -> tuple[cv2.typing.Scalar, cv2.typing.MatLike]: ...
    @_typing.overload
    def compute(self, ref: cv2.UMat, cmp: cv2.UMat, qualityMap: cv2.UMat | None = ...) -> tuple[cv2.typing.Scalar, cv2.UMat]: ...

    def empty(self) -> bool: ...

    def clear(self) -> None: ...

    @classmethod
    @_typing.overload
    def create(cls, ref: cv2.typing.MatLike) -> QualityMSE: ...
    @classmethod
    @_typing.overload
    def create(cls, ref: cv2.UMat) -> QualityMSE: ...


class QualityPSNR(QualityBase):
    # Functions
    @classmethod
    @_typing.overload
    def create(cls, ref: cv2.typing.MatLike, maxPixelValue: float = ...) -> QualityPSNR: ...
    @classmethod
    @_typing.overload
    def create(cls, ref: cv2.UMat, maxPixelValue: float = ...) -> QualityPSNR: ...

    @_typing.overload
    def compute(self, cmp: cv2.typing.MatLike) -> tuple[float, float, float, float]: ...
    @_typing.overload
    def compute(self, cmp: cv2.UMat) -> tuple[float, float, float, float]: ...
    @_typing.overload
    def compute(self, ref: cv2.typing.MatLike, cmp: cv2.typing.MatLike, qualityMap: cv2.typing.MatLike | None = ..., maxPixelValue: float = ...) -> tuple[cv2.typing.Scalar, cv2.typing.MatLike]: ...
    @_typing.overload
    def compute(self, ref: cv2.UMat, cmp: cv2.UMat, qualityMap: cv2.UMat | None = ..., maxPixelValue: float = ...) -> tuple[cv2.typing.Scalar, cv2.UMat]: ...

    def empty(self) -> bool: ...

    def clear(self) -> None: ...

    def getMaxPixelValue(self) -> float: ...

    def setMaxPixelValue(self, val: float) -> None: ...


class QualitySSIM(QualityBase):
    # Functions
    @_typing.overload
    def compute(self, cmp: cv2.typing.MatLike) -> tuple[float, float, float, float]: ...
    @_typing.overload
    def compute(self, cmp: cv2.UMat) -> tuple[float, float, float, float]: ...
    @_typing.overload
    def compute(self, ref: cv2.typing.MatLike, cmp: cv2.typing.MatLike, qualityMap: cv2.typing.MatLike | None = ...) -> tuple[cv2.typing.Scalar, cv2.typing.MatLike]: ...
    @_typing.overload
    def compute(self, ref: cv2.UMat, cmp: cv2.UMat, qualityMap: cv2.UMat | None = ...) -> tuple[cv2.typing.Scalar, cv2.UMat]: ...

    def empty(self) -> bool: ...

    def clear(self) -> None: ...

    @classmethod
    @_typing.overload
    def create(cls, ref: cv2.typing.MatLike) -> QualitySSIM: ...
    @classmethod
    @_typing.overload
    def create(cls, ref: cv2.UMat) -> QualitySSIM: ...






#!/usr/bin/env python

'''
camera calibration for distorted images with chess board samples
reads distorted images, calculates the calibration and write undistorted images

usage:
    calibrate.py [--debug <output path>] [-w <width>] [-h <height>] [-t <pattern type>] [--square_size=<square size>]
    [--marker_size=<aruco marker size>] [--aruco_dict=<aruco dictionary name>] [<image mask>]

usage example:
    calibrate.py -w 4 -h 6 -t chessboard --square_size=50 ../data/left*.jpg

default values:
    --debug:    ./output/
    -w: 4
    -h: 6
    -t: chessboard
    --square_size: 10
    --marker_size: 5
    --aruco_dict: DICT_4X4_50
    --threads: 4
    <image mask> defaults to ../data/left*.jpg

NOTE: Chessboard size is defined in inner corners. Charuco board size is defined in units.
'''

# Python 2/3 compatibility
from __future__ import print_function

import numpy as np
import cv2 as cv

# local modules
from common import splitfn

# built-in modules
import os

def main():
    import sys
    import getopt
    from glob import glob

    args, img_names = getopt.getopt(sys.argv[1:], 'w:h:t:', ['debug=','square_size=', 'marker_size=',
                                                      'aruco_dict=', 'threads=', ])
    args = dict(args)
    args.setdefault('--debug', './output/')
    args.setdefault('-w', 4)
    args.setdefault('-h', 6)
    args.setdefault('-t', 'chessboard')
    args.setdefault('--square_size', 10)
    args.setdefault('--marker_size', 5)
    args.setdefault('--aruco_dict', 'DICT_4X4_50')
    args.setdefault('--threads', 4)

    if not img_names:
        img_mask = '../data/left??.jpg'  # default
        img_names = glob(img_mask)

    debug_dir = args.get('--debug')
    if debug_dir and not os.path.isdir(debug_dir):
        os.mkdir(debug_dir)

    height = int(args.get('-h'))
    width = int(args.get('-w'))
    pattern_type = str(args.get('-t'))
    square_size = float(args.get('--square_size'))
    marker_size = float(args.get('--marker_size'))
    aruco_dict_name = str(args.get('--aruco_dict'))

    pattern_size = (width, height)
    if pattern_type == 'chessboard':
        pattern_points = np.zeros((np.prod(pattern_size), 3), np.float32)
        pattern_points[:, :2] = np.indices(pattern_size).T.reshape(-1, 2)
        pattern_points *= square_size

    obj_points = []
    img_points = []
    h, w = cv.imread(img_names[0], cv.IMREAD_GRAYSCALE).shape[:2]  # TODO: use imquery call to retrieve results

    aruco_dicts = {
        'DICT_4X4_50': cv.aruco.DICT_4X4_50,
        'DICT_4X4_100': cv.aruco.DICT_4X4_100,
        'DICT_4X4_250': cv.aruco.DICT_4X4_250,
        'DICT_4X4_1000': cv.aruco.DICT_4X4_1000,
        'DICT_5X5_50': cv.aruco.DICT_5X5_50,
        'DICT_5X5_100': cv.aruco.DICT_5X5_100,
        'DICT_5X5_250': cv.aruco.DICT_5X5_250,
        'DICT_5X5_1000': cv.aruco.DICT_5X5_1000,
        'DICT_6X6_50': cv.aruco.DICT_6X6_50,
        'DICT_6X6_100': cv.aruco.DICT_6X6_100,
        'DICT_6X6_250': cv.aruco.DICT_6X6_250,
        'DICT_6X6_1000': cv.aruco.DICT_6X6_1000,
        'DICT_7X7_50': cv.aruco.DICT_7X7_50,
        'DICT_7X7_100': cv.aruco.DICT_7X7_100,
        'DICT_7X7_250': cv.aruco.DICT_7X7_250,
        'DICT_7X7_1000': cv.aruco.DICT_7X7_1000,
        'DICT_ARUCO_ORIGINAL': cv.aruco.DICT_ARUCO_ORIGINAL,
        'DICT_APRILTAG_16h5': cv.aruco.DICT_APRILTAG_16h5,
        'DICT_APRILTAG_25h9': cv.aruco.DICT_APRILTAG_25h9,
        'DICT_APRILTAG_36h10': cv.aruco.DICT_APRILTAG_36h10,
        'DICT_APRILTAG_36h11': cv.aruco.DICT_APRILTAG_36h11,
        'DICT_ARUCO_MIP_36h12': cv.aruco.DICT_ARUCO_MIP_36h12
    }

    if (aruco_dict_name not in set(aruco_dicts.keys())):
        print("unknown aruco dictionary name")
        return None
    aruco_dict = cv.aruco.getPredefinedDictionary(aruco_dicts[aruco_dict_name])
    board = cv.aruco.CharucoBoard(pattern_size, square_size, marker_size, aruco_dict)
    charuco_detector = cv.aruco.CharucoDetector(board)

    def processImage(fn):
        print('processing %s... ' % fn)
        img = cv.imread(fn, cv.IMREAD_GRAYSCALE)
        if img is None:
            print("Failed to load", fn)
            return None

        assert w == img.shape[1] and h == img.shape[0], ("size: %d x %d ... " % (img.shape[1], img.shape[0]))
        found = False
        corners = 0
        if pattern_type == 'chessboard':
            found, corners = cv.findChessboardCorners(img, pattern_size)
            if found:
                term = (cv.TERM_CRITERIA_EPS + cv.TERM_CRITERIA_COUNT, 30, 0.1)
                cv.cornerSubPix(img, corners, (5, 5), (-1, -1), term)
                frame_img_points = corners.reshape(-1, 2)
                frame_obj_points = pattern_points
        elif pattern_type == 'charucoboard':
            corners, charucoIds, _, _ = charuco_detector.detectBoard(img)
            if (len(corners) > 0):
                frame_obj_points, frame_img_points = board.matchImagePoints(corners, charucoIds)
                found = True
            else:
                found = False
        else:
            print("unknown pattern type", pattern_type)
            return None

        if debug_dir:
            vis = cv.cvtColor(img, cv.COLOR_GRAY2BGR)
            if pattern_type == 'chessboard':
                cv.drawChessboardCorners(vis, pattern_size, corners, found)
            elif pattern_type == 'charucoboard':
                cv.aruco.drawDetectedCornersCharuco(vis, corners, charucoIds=charucoIds)
            _path, name, _ext = splitfn(fn)
            outfile = os.path.join(debug_dir, name + '_board.png')
            cv.imwrite(outfile, vis)

        if not found:
            print('pattern not found')
            return None

        print('           %s... OK' % fn)
        return (frame_img_points, frame_obj_points)

    threads_num = int(args.get('--threads'))
    if threads_num <= 1:
        chessboards = [processImage(fn) for fn in img_names]
    else:
        print("Run with %d threads..." % threads_num)
        from multiprocessing.dummy import Pool as ThreadPool
        pool = ThreadPool(threads_num)
        chessboards = pool.map(processImage, img_names)

    chessboards = [x for x in chessboards if x is not None]
    for (corners, pattern_points) in chessboards:
        img_points.append(corners)
        obj_points.append(pattern_points)

    # calculate camera distortion
    rms, camera_matrix, dist_coefs, _rvecs, _tvecs = cv.calibrateCamera(obj_points, img_points, (w, h), None, None)

    print("\nRMS:", rms)
    print("camera matrix:\n", camera_matrix)
    print("distortion coefficients: ", dist_coefs.ravel())

    # undistort the image with the calibration
    print('')
    for fn in img_names if debug_dir else []:
        _path, name, _ext = splitfn(fn)
        img_found = os.path.join(debug_dir, name + '_board.png')
        outfile = os.path.join(debug_dir, name + '_undistorted.png')

        img = cv.imread(img_found)
        if img is None:
            continue

        h, w = img.shape[:2]
        newcameramtx, roi = cv.getOptimalNewCameraMatrix(camera_matrix, dist_coefs, (w, h), 1, (w, h))

        dst = cv.undistort(img, camera_matrix, dist_coefs, None, newcameramtx)

        # crop and save the image
        x, y, w, h = roi
        dst = dst[y:y+h, x:x+w]

        print('Undistorted image written to: %s' % outfile)
        cv.imwrite(outfile, dst)

    print('Done')


if __name__ == '__main__':
    print(__doc__)
    main()
    cv.destroyAllWindows()



__all__: list[str] = []

import cv2
import cv2.dnn
import cv2.typing
import typing as _typing


# Enumerations
MCC24: int
SG140: int
VINYL18: int
TYPECHART = int
"""One of [MCC24, SG140, VINYL18]"""



# Classes
class DetectorParameters:
    adaptiveThreshWinSizeMin: int
    adaptiveThreshWinSizeMax: int
    adaptiveThreshWinSizeStep: int
    adaptiveThreshConstant: float
    minContoursAreaRate: float
    minContoursArea: float
    confidenceThreshold: float
    minContourSolidity: float
    findCandidatesApproxPolyDPEpsMultiplier: float
    borderWidth: int
    B0factor: float
    maxError: float
    minContourPointsAllowed: int
    minContourLengthAllowed: int
    minInterContourDistance: int
    minInterCheckerDistance: int
    minImageSize: int
    minGroupSize: int

    # Functions
    @classmethod
    def create(cls) -> DetectorParameters: ...


class CCheckerDetector(cv2.Algorithm):
    # Functions
    def setNet(self, net: cv2.dnn.Net) -> bool: ...

    @_typing.overload
    def processWithROI(self, image: cv2.typing.MatLike, chartType: TYPECHART, regionsOfInterest: _typing.Sequence[cv2.typing.Rect], nc: int = ..., useNet: bool = ..., params: DetectorParameters = ...) -> bool: ...
    @_typing.overload
    def processWithROI(self, image: cv2.UMat, chartType: TYPECHART, regionsOfInterest: _typing.Sequence[cv2.typing.Rect], nc: int = ..., useNet: bool = ..., params: DetectorParameters = ...) -> bool: ...

    @_typing.overload
    def process(self, image: cv2.typing.MatLike, chartType: TYPECHART, nc: int = ..., useNet: bool = ..., params: DetectorParameters = ...) -> bool: ...
    @_typing.overload
    def process(self, image: cv2.UMat, chartType: TYPECHART, nc: int = ..., useNet: bool = ..., params: DetectorParameters = ...) -> bool: ...

    def getBestColorChecker(self) -> CChecker: ...

    def getListColorChecker(self) -> _typing.Sequence[CChecker]: ...

    @classmethod
    def create(cls) -> CCheckerDetector: ...


class CChecker:
    # Functions
    @classmethod
    def create(cls) -> CChecker: ...

    def setTarget(self, _target: TYPECHART) -> None: ...

    def setBox(self, _box: _typing.Sequence[cv2.typing.Point2f]) -> None: ...

    def setChartsRGB(self, _chartsRGB: cv2.typing.MatLike) -> None: ...

    def setChartsYCbCr(self, _chartsYCbCr: cv2.typing.MatLike) -> None: ...

    def setCost(self, _cost: float) -> None: ...

    def setCenter(self, _center: cv2.typing.Point2f) -> None: ...

    def getTarget(self) -> TYPECHART: ...

    def getBox(self) -> _typing.Sequence[cv2.typing.Point2f]: ...

    def getColorCharts(self) -> _typing.Sequence[cv2.typing.Point2f]: ...

    def getChartsRGB(self) -> cv2.typing.MatLike: ...

    def getChartsYCbCr(self) -> cv2.typing.MatLike: ...

    def getCost(self) -> float: ...

    def getCenter(self) -> cv2.typing.Point2f: ...


class CCheckerDraw:
    # Functions
    @_typing.overload
    def draw(self, img: cv2.typing.MatLike) -> cv2.typing.MatLike: ...
    @_typing.overload
    def draw(self, img: cv2.UMat) -> cv2.UMat: ...

    @classmethod
    def create(cls, pChecker: CChecker, color: cv2.typing.Scalar = ..., thickness: int = ...) -> CCheckerDraw: ...






__all__: list[str] = []

import cv2
import cv2.typing
import typing as _typing


from cv2.ximgproc import segmentation as segmentation


# Enumerations
THINNING_ZHANGSUEN: int
THINNING_GUOHALL: int
ThinningTypes = int
"""One of [THINNING_ZHANGSUEN, THINNING_GUOHALL]"""

BINARIZATION_NIBLACK: int
BINARIZATION_SAUVOLA: int
BINARIZATION_WOLF: int
BINARIZATION_NICK: int
LocalBinarizationMethods = int
"""One of [BINARIZATION_NIBLACK, BINARIZATION_SAUVOLA, BINARIZATION_WOLF, BINARIZATION_NICK]"""

DTF_NC: int
DTF_IC: int
DTF_RF: int
GUIDED_FILTER: int
AM_FILTER: int
EdgeAwareFiltersList = int
"""One of [DTF_NC, DTF_IC, DTF_RF, GUIDED_FILTER, AM_FILTER]"""

ARO_0_45: int
ARO_45_90: int
ARO_90_135: int
ARO_315_0: int
ARO_315_45: int
ARO_45_135: int
ARO_315_135: int
ARO_CTR_HOR: int
ARO_CTR_VER: int
AngleRangeOption = int
"""One of [ARO_0_45, ARO_45_90, ARO_90_135, ARO_315_0, ARO_315_45, ARO_45_135, ARO_315_135, ARO_CTR_HOR, ARO_CTR_VER]"""

FHT_MIN: int
FHT_MAX: int
FHT_ADD: int
FHT_AVE: int
HoughOp = int
"""One of [FHT_MIN, FHT_MAX, FHT_ADD, FHT_AVE]"""

HDO_RAW: int
HDO_DESKEW: int
HoughDeskewOption = int
"""One of [HDO_RAW, HDO_DESKEW]"""

SLIC: int
SLICO: int
MSLIC: int
SLICType = int
"""One of [SLIC, SLICO, MSLIC]"""

WMF_EXP: int
WMF_IV1: int
WMF_IV2: int
WMF_COS: int
WMF_JAC: int
WMF_OFF: int
WMFWeightType = int
"""One of [WMF_EXP, WMF_IV1, WMF_IV2, WMF_COS, WMF_JAC, WMF_OFF]"""


EdgeDrawing_PREWITT: int
EDGE_DRAWING_PREWITT: int
EdgeDrawing_SOBEL: int
EDGE_DRAWING_SOBEL: int
EdgeDrawing_SCHARR: int
EDGE_DRAWING_SCHARR: int
EdgeDrawing_LSD: int
EDGE_DRAWING_LSD: int
EdgeDrawing_GradientOperator = int
"""One of [EdgeDrawing_PREWITT, EDGE_DRAWING_PREWITT, EdgeDrawing_SOBEL, EDGE_DRAWING_SOBEL, EdgeDrawing_SCHARR, EDGE_DRAWING_SCHARR, EdgeDrawing_LSD, EDGE_DRAWING_LSD]"""


# Classes
class DisparityFilter(cv2.Algorithm):
    # Functions
    @_typing.overload
    def filter(self, disparity_map_left: cv2.typing.MatLike, left_view: cv2.typing.MatLike, filtered_disparity_map: cv2.typing.MatLike | None = ..., disparity_map_right: cv2.typing.MatLike | None = ..., ROI: cv2.typing.Rect = ..., right_view: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
    @_typing.overload
    def filter(self, disparity_map_left: cv2.UMat, left_view: cv2.UMat, filtered_disparity_map: cv2.UMat | None = ..., disparity_map_right: cv2.UMat | None = ..., ROI: cv2.typing.Rect = ..., right_view: cv2.UMat | None = ...) -> cv2.UMat: ...


class DisparityWLSFilter(DisparityFilter):
    # Functions
    def getLambda(self) -> float: ...

    def setLambda(self, _lambda: float) -> None: ...

    def getSigmaColor(self) -> float: ...

    def setSigmaColor(self, _sigma_color: float) -> None: ...

    def getLRCthresh(self) -> int: ...

    def setLRCthresh(self, _LRC_thresh: int) -> None: ...

    def getDepthDiscontinuityRadius(self) -> int: ...

    def setDepthDiscontinuityRadius(self, _disc_radius: int) -> None: ...

    def getConfidenceMap(self) -> cv2.typing.MatLike: ...

    def getROI(self) -> cv2.typing.Rect: ...


class EdgeDrawing(cv2.Algorithm):
    # Classes
    class Params:
        PFmode: bool
        EdgeDetectionOperator: int
        GradientThresholdValue: int
        AnchorThresholdValue: int
        ScanInterval: int
        MinPathLength: int
        Sigma: float
        SumFlag: bool
        NFAValidation: bool
        MinLineLength: int
        MaxDistanceBetweenTwoLines: float
        LineFitErrorThreshold: float
        MaxErrorThreshold: float

        # Functions
        def __init__(self) -> None: ...

        def read(self, fn: cv2.FileNode) -> None: ...

        def write(self, fs: cv2.FileStorage) -> None: ...



    # Functions
    @_typing.overload
    def detectEdges(self, src: cv2.typing.MatLike) -> None: ...
    @_typing.overload
    def detectEdges(self, src: cv2.UMat) -> None: ...

    @_typing.overload
    def getEdgeImage(self, dst: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
    @_typing.overload
    def getEdgeImage(self, dst: cv2.UMat | None = ...) -> cv2.UMat: ...

    @_typing.overload
    def getGradientImage(self, dst: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
    @_typing.overload
    def getGradientImage(self, dst: cv2.UMat | None = ...) -> cv2.UMat: ...

    def getSegments(self) -> _typing.Sequence[_typing.Sequence[cv2.typing.Point]]: ...

    def getSegmentIndicesOfLines(self) -> _typing.Sequence[int]: ...

    @_typing.overload
    def detectLines(self, lines: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
    @_typing.overload
    def detectLines(self, lines: cv2.UMat | None = ...) -> cv2.UMat: ...

    @_typing.overload
    def detectEllipses(self, ellipses: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
    @_typing.overload
    def detectEllipses(self, ellipses: cv2.UMat | None = ...) -> cv2.UMat: ...

    def setParams(self, parameters: EdgeDrawing.Params) -> None: ...


class DTFilter(cv2.Algorithm):
    # Functions
    @_typing.overload
    def filter(self, src: cv2.typing.MatLike, dst: cv2.typing.MatLike | None = ..., dDepth: int = ...) -> cv2.typing.MatLike: ...
    @_typing.overload
    def filter(self, src: cv2.UMat, dst: cv2.UMat | None = ..., dDepth: int = ...) -> cv2.UMat: ...


class GuidedFilter(cv2.Algorithm):
    # Functions
    @_typing.overload
    def filter(self, src: cv2.typing.MatLike, dst: cv2.typing.MatLike | None = ..., dDepth: int = ...) -> cv2.typing.MatLike: ...
    @_typing.overload
    def filter(self, src: cv2.UMat, dst: cv2.UMat | None = ..., dDepth: int = ...) -> cv2.UMat: ...


class AdaptiveManifoldFilter(cv2.Algorithm):
    # Functions
    @_typing.overload
    def filter(self, src: cv2.typing.MatLike, dst: cv2.typing.MatLike | None = ..., joint: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
    @_typing.overload
    def filter(self, src: cv2.UMat, dst: cv2.UMat | None = ..., joint: cv2.UMat | None = ...) -> cv2.UMat: ...

    def collectGarbage(self) -> None: ...

    @classmethod
    def create(cls) -> AdaptiveManifoldFilter: ...


class FastBilateralSolverFilter(cv2.Algorithm):
    # Functions
    @_typing.overload
    def filter(self, src: cv2.typing.MatLike, confidence: cv2.typing.MatLike, dst: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
    @_typing.overload
    def filter(self, src: cv2.UMat, confidence: cv2.UMat, dst: cv2.UMat | None = ...) -> cv2.UMat: ...


class FastGlobalSmootherFilter(cv2.Algorithm):
    # Functions
    @_typing.overload
    def filter(self, src: cv2.typing.MatLike, dst: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
    @_typing.overload
    def filter(self, src: cv2.UMat, dst: cv2.UMat | None = ...) -> cv2.UMat: ...


class EdgeBoxes(cv2.Algorithm):
    # Functions
    @_typing.overload
    def getBoundingBoxes(self, edge_map: cv2.typing.MatLike, orientation_map: cv2.typing.MatLike, scores: cv2.typing.MatLike | None = ...) -> tuple[_typing.Sequence[cv2.typing.Rect], cv2.typing.MatLike]: ...
    @_typing.overload
    def getBoundingBoxes(self, edge_map: cv2.UMat, orientation_map: cv2.UMat, scores: cv2.UMat | None = ...) -> tuple[_typing.Sequence[cv2.typing.Rect], cv2.UMat]: ...

    def getAlpha(self) -> float: ...

    def setAlpha(self, value: float) -> None: ...

    def getBeta(self) -> float: ...

    def setBeta(self, value: float) -> None: ...

    def getEta(self) -> float: ...

    def setEta(self, value: float) -> None: ...

    def getMinScore(self) -> float: ...

    def setMinScore(self, value: float) -> None: ...

    def getMaxBoxes(self) -> int: ...

    def setMaxBoxes(self, value: int) -> None: ...

    def getEdgeMinMag(self) -> float: ...

    def setEdgeMinMag(self, value: float) -> None: ...

    def getEdgeMergeThr(self) -> float: ...

    def setEdgeMergeThr(self, value: float) -> None: ...

    def getClusterMinMag(self) -> float: ...

    def setClusterMinMag(self, value: float) -> None: ...

    def getMaxAspectRatio(self) -> float: ...

    def setMaxAspectRatio(self, value: float) -> None: ...

    def getMinBoxArea(self) -> float: ...

    def setMinBoxArea(self, value: float) -> None: ...

    def getGamma(self) -> float: ...

    def setGamma(self, value: float) -> None: ...

    def getKappa(self) -> float: ...

    def setKappa(self, value: float) -> None: ...


class FastLineDetector(cv2.Algorithm):
    # Functions
    @_typing.overload
    def detect(self, image: cv2.typing.MatLike, lines: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
    @_typing.overload
    def detect(self, image: cv2.UMat, lines: cv2.UMat | None = ...) -> cv2.UMat: ...

    @_typing.overload
    def drawSegments(self, image: cv2.typing.MatLike, lines: cv2.typing.MatLike, draw_arrow: bool = ..., linecolor: cv2.typing.Scalar = ..., linethickness: int = ...) -> cv2.typing.MatLike: ...
    @_typing.overload
    def drawSegments(self, image: cv2.UMat, lines: cv2.UMat, draw_arrow: bool = ..., linecolor: cv2.typing.Scalar = ..., linethickness: int = ...) -> cv2.UMat: ...


class ContourFitting(cv2.Algorithm):
    # Functions
    @_typing.overload
    def estimateTransformation(self, src: cv2.typing.MatLike, dst: cv2.typing.MatLike, alphaPhiST: cv2.typing.MatLike | None = ..., fdContour: bool = ...) -> tuple[cv2.typing.MatLike, float]: ...
    @_typing.overload
    def estimateTransformation(self, src: cv2.UMat, dst: cv2.UMat, alphaPhiST: cv2.UMat | None = ..., fdContour: bool = ...) -> tuple[cv2.UMat, float]: ...

    def setCtrSize(self, n: int) -> None: ...

    def setFDSize(self, n: int) -> None: ...

    def getCtrSize(self) -> int: ...

    def getFDSize(self) -> int: ...


class SuperpixelLSC(cv2.Algorithm):
    # Functions
    def getNumberOfSuperpixels(self) -> int: ...

    def iterate(self, num_iterations: int = ...) -> None: ...

    @_typing.overload
    def getLabels(self, labels_out: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
    @_typing.overload
    def getLabels(self, labels_out: cv2.UMat | None = ...) -> cv2.UMat: ...

    @_typing.overload
    def getLabelContourMask(self, image: cv2.typing.MatLike | None = ..., thick_line: bool = ...) -> cv2.typing.MatLike: ...
    @_typing.overload
    def getLabelContourMask(self, image: cv2.UMat | None = ..., thick_line: bool = ...) -> cv2.UMat: ...

    def enforceLabelConnectivity(self, min_element_size: int = ...) -> None: ...


class RidgeDetectionFilter(cv2.Algorithm):
    # Functions
    @classmethod
    def create(cls, ddepth: int = ..., dx: int = ..., dy: int = ..., ksize: int = ..., out_dtype: int = ..., scale: float = ..., delta: float = ..., borderType: int = ...) -> RidgeDetectionFilter: ...

    @_typing.overload
    def getRidgeFilteredImage(self, _img: cv2.typing.MatLike, out: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
    @_typing.overload
    def getRidgeFilteredImage(self, _img: cv2.UMat, out: cv2.UMat | None = ...) -> cv2.UMat: ...


class ScanSegment(cv2.Algorithm):
    # Functions
    def getNumberOfSuperpixels(self) -> int: ...

    @_typing.overload
    def iterate(self, img: cv2.typing.MatLike) -> None: ...
    @_typing.overload
    def iterate(self, img: cv2.UMat) -> None: ...

    @_typing.overload
    def getLabels(self, labels_out: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
    @_typing.overload
    def getLabels(self, labels_out: cv2.UMat | None = ...) -> cv2.UMat: ...

    @_typing.overload
    def getLabelContourMask(self, image: cv2.typing.MatLike | None = ..., thick_line: bool = ...) -> cv2.typing.MatLike: ...
    @_typing.overload
    def getLabelContourMask(self, image: cv2.UMat | None = ..., thick_line: bool = ...) -> cv2.UMat: ...


class SuperpixelSEEDS(cv2.Algorithm):
    # Functions
    def getNumberOfSuperpixels(self) -> int: ...

    @_typing.overload
    def iterate(self, img: cv2.typing.MatLike, num_iterations: int = ...) -> None: ...
    @_typing.overload
    def iterate(self, img: cv2.UMat, num_iterations: int = ...) -> None: ...

    @_typing.overload
    def getLabels(self, labels_out: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
    @_typing.overload
    def getLabels(self, labels_out: cv2.UMat | None = ...) -> cv2.UMat: ...

    @_typing.overload
    def getLabelContourMask(self, image: cv2.typing.MatLike | None = ..., thick_line: bool = ...) -> cv2.typing.MatLike: ...
    @_typing.overload
    def getLabelContourMask(self, image: cv2.UMat | None = ..., thick_line: bool = ...) -> cv2.UMat: ...


class SuperpixelSLIC(cv2.Algorithm):
    # Functions
    def getNumberOfSuperpixels(self) -> int: ...

    def iterate(self, num_iterations: int = ...) -> None: ...

    @_typing.overload
    def getLabels(self, labels_out: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
    @_typing.overload
    def getLabels(self, labels_out: cv2.UMat | None = ...) -> cv2.UMat: ...

    @_typing.overload
    def getLabelContourMask(self, image: cv2.typing.MatLike | None = ..., thick_line: bool = ...) -> cv2.typing.MatLike: ...
    @_typing.overload
    def getLabelContourMask(self, image: cv2.UMat | None = ..., thick_line: bool = ...) -> cv2.UMat: ...

    def enforceLabelConnectivity(self, min_element_size: int = ...) -> None: ...


class SparseMatchInterpolator(cv2.Algorithm):
    # Functions
    @_typing.overload
    def interpolate(self, from_image: cv2.typing.MatLike, from_points: cv2.typing.MatLike, to_image: cv2.typing.MatLike, to_points: cv2.typing.MatLike, dense_flow: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
    @_typing.overload
    def interpolate(self, from_image: cv2.UMat, from_points: cv2.UMat, to_image: cv2.UMat, to_points: cv2.UMat, dense_flow: cv2.UMat | None = ...) -> cv2.UMat: ...


class EdgeAwareInterpolator(SparseMatchInterpolator):
    # Functions
    def setCostMap(self, _costMap: cv2.typing.MatLike) -> None: ...

    def setK(self, _k: int) -> None: ...

    def getK(self) -> int: ...

    def setSigma(self, _sigma: float) -> None: ...

    def getSigma(self) -> float: ...

    def setLambda(self, _lambda: float) -> None: ...

    def getLambda(self) -> float: ...

    def setUsePostProcessing(self, _use_post_proc: bool) -> None: ...

    def getUsePostProcessing(self) -> bool: ...

    def setFGSLambda(self, _lambda: float) -> None: ...

    def getFGSLambda(self) -> float: ...

    def setFGSSigma(self, _sigma: float) -> None: ...

    def getFGSSigma(self) -> float: ...


class RICInterpolator(SparseMatchInterpolator):
    # Functions
    def setK(self, k: int = ...) -> None: ...

    def getK(self) -> int: ...

    def setCostMap(self, costMap: cv2.typing.MatLike) -> None: ...

    def setSuperpixelSize(self, spSize: int = ...) -> None: ...

    def getSuperpixelSize(self) -> int: ...

    def setSuperpixelNNCnt(self, spNN: int = ...) -> None: ...

    def getSuperpixelNNCnt(self) -> int: ...

    def setSuperpixelRuler(self, ruler: float = ...) -> None: ...

    def getSuperpixelRuler(self) -> float: ...

    def setSuperpixelMode(self, mode: int = ...) -> None: ...

    def getSuperpixelMode(self) -> int: ...

    def setAlpha(self, alpha: float = ...) -> None: ...

    def getAlpha(self) -> float: ...

    def setModelIter(self, modelIter: int = ...) -> None: ...

    def getModelIter(self) -> int: ...

    def setRefineModels(self, refineModles: bool = ...) -> None: ...

    def getRefineModels(self) -> bool: ...

    def setMaxFlow(self, maxFlow: float = ...) -> None: ...

    def getMaxFlow(self) -> float: ...

    def setUseVariationalRefinement(self, use_variational_refinement: bool = ...) -> None: ...

    def getUseVariationalRefinement(self) -> bool: ...

    def setUseGlobalSmootherFilter(self, use_FGS: bool = ...) -> None: ...

    def getUseGlobalSmootherFilter(self) -> bool: ...

    def setFGSLambda(self, lambda_: float = ...) -> None: ...

    def getFGSLambda(self) -> float: ...

    def setFGSSigma(self, sigma: float = ...) -> None: ...

    def getFGSSigma(self) -> float: ...


class RFFeatureGetter(cv2.Algorithm):
    # Functions
    def getFeatures(self, src: cv2.typing.MatLike, features: cv2.typing.MatLike, gnrmRad: int, gsmthRad: int, shrink: int, outNum: int, gradNum: int) -> None: ...


class StructuredEdgeDetection(cv2.Algorithm):
    # Functions
    @_typing.overload
    def detectEdges(self, src: cv2.typing.MatLike, dst: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
    @_typing.overload
    def detectEdges(self, src: cv2.UMat, dst: cv2.UMat | None = ...) -> cv2.UMat: ...

    @_typing.overload
    def computeOrientation(self, src: cv2.typing.MatLike, dst: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
    @_typing.overload
    def computeOrientation(self, src: cv2.UMat, dst: cv2.UMat | None = ...) -> cv2.UMat: ...

    @_typing.overload
    def edgesNms(self, edge_image: cv2.typing.MatLike, orientation_image: cv2.typing.MatLike, dst: cv2.typing.MatLike | None = ..., r: int = ..., s: int = ..., m: float = ..., isParallel: bool = ...) -> cv2.typing.MatLike: ...
    @_typing.overload
    def edgesNms(self, edge_image: cv2.UMat, orientation_image: cv2.UMat, dst: cv2.UMat | None = ..., r: int = ..., s: int = ..., m: float = ..., isParallel: bool = ...) -> cv2.UMat: ...



# Functions
@_typing.overload
def FastHoughTransform(src: cv2.typing.MatLike, dstMatDepth: int, dst: cv2.typing.MatLike | None = ..., angleRange: int = ..., op: int = ..., makeSkew: int = ...) -> cv2.typing.MatLike: ...
@_typing.overload
def FastHoughTransform(src: cv2.UMat, dstMatDepth: int, dst: cv2.UMat | None = ..., angleRange: int = ..., op: int = ..., makeSkew: int = ...) -> cv2.UMat: ...

@_typing.overload
def GradientDericheX(op: cv2.typing.MatLike, alpha: float, omega: float, dst: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
@_typing.overload
def GradientDericheX(op: cv2.UMat, alpha: float, omega: float, dst: cv2.UMat | None = ...) -> cv2.UMat: ...

@_typing.overload
def GradientDericheY(op: cv2.typing.MatLike, alpha: float, omega: float, dst: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
@_typing.overload
def GradientDericheY(op: cv2.UMat, alpha: float, omega: float, dst: cv2.UMat | None = ...) -> cv2.UMat: ...

@_typing.overload
def HoughPoint2Line(houghPoint: cv2.typing.Point, srcImgInfo: cv2.typing.MatLike, angleRange: int = ..., makeSkew: int = ..., rules: int = ...) -> cv2.typing.Vec4i: ...
@_typing.overload
def HoughPoint2Line(houghPoint: cv2.typing.Point, srcImgInfo: cv2.UMat, angleRange: int = ..., makeSkew: int = ..., rules: int = ...) -> cv2.typing.Vec4i: ...

@_typing.overload
def PeiLinNormalization(I: cv2.typing.MatLike, T: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
@_typing.overload
def PeiLinNormalization(I: cv2.UMat, T: cv2.UMat | None = ...) -> cv2.UMat: ...

@_typing.overload
def RadonTransform(src: cv2.typing.MatLike, dst: cv2.typing.MatLike | None = ..., theta: float = ..., start_angle: float = ..., end_angle: float = ..., crop: bool = ..., norm: bool = ...) -> cv2.typing.MatLike: ...
@_typing.overload
def RadonTransform(src: cv2.UMat, dst: cv2.UMat | None = ..., theta: float = ..., start_angle: float = ..., end_angle: float = ..., crop: bool = ..., norm: bool = ...) -> cv2.UMat: ...

@_typing.overload
def amFilter(joint: cv2.typing.MatLike, src: cv2.typing.MatLike, sigma_s: float, sigma_r: float, dst: cv2.typing.MatLike | None = ..., adjust_outliers: bool = ...) -> cv2.typing.MatLike: ...
@_typing.overload
def amFilter(joint: cv2.UMat, src: cv2.UMat, sigma_s: float, sigma_r: float, dst: cv2.UMat | None = ..., adjust_outliers: bool = ...) -> cv2.UMat: ...

@_typing.overload
def anisotropicDiffusion(src: cv2.typing.MatLike, alpha: float, K: float, niters: int, dst: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
@_typing.overload
def anisotropicDiffusion(src: cv2.UMat, alpha: float, K: float, niters: int, dst: cv2.UMat | None = ...) -> cv2.UMat: ...

@_typing.overload
def bilateralTextureFilter(src: cv2.typing.MatLike, dst: cv2.typing.MatLike | None = ..., fr: int = ..., numIter: int = ..., sigmaAlpha: float = ..., sigmaAvg: float = ...) -> cv2.typing.MatLike: ...
@_typing.overload
def bilateralTextureFilter(src: cv2.UMat, dst: cv2.UMat | None = ..., fr: int = ..., numIter: int = ..., sigmaAlpha: float = ..., sigmaAvg: float = ...) -> cv2.UMat: ...

@_typing.overload
def colorMatchTemplate(img: cv2.typing.MatLike, templ: cv2.typing.MatLike, result: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
@_typing.overload
def colorMatchTemplate(img: cv2.UMat, templ: cv2.UMat, result: cv2.UMat | None = ...) -> cv2.UMat: ...

@_typing.overload
def computeBadPixelPercent(GT: cv2.typing.MatLike, src: cv2.typing.MatLike, ROI: cv2.typing.Rect, thresh: int = ...) -> float: ...
@_typing.overload
def computeBadPixelPercent(GT: cv2.UMat, src: cv2.UMat, ROI: cv2.typing.Rect, thresh: int = ...) -> float: ...

@_typing.overload
def computeMSE(GT: cv2.typing.MatLike, src: cv2.typing.MatLike, ROI: cv2.typing.Rect) -> float: ...
@_typing.overload
def computeMSE(GT: cv2.UMat, src: cv2.UMat, ROI: cv2.typing.Rect) -> float: ...

@_typing.overload
def contourSampling(src: cv2.typing.MatLike, nbElt: int, out: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
@_typing.overload
def contourSampling(src: cv2.UMat, nbElt: int, out: cv2.UMat | None = ...) -> cv2.UMat: ...

@_typing.overload
def covarianceEstimation(src: cv2.typing.MatLike, windowRows: int, windowCols: int, dst: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
@_typing.overload
def covarianceEstimation(src: cv2.UMat, windowRows: int, windowCols: int, dst: cv2.UMat | None = ...) -> cv2.UMat: ...

def createAMFilter(sigma_s: float, sigma_r: float, adjust_outliers: bool = ...) -> AdaptiveManifoldFilter: ...

def createContourFitting(ctr: int = ..., fd: int = ...) -> ContourFitting: ...

@_typing.overload
def createDTFilter(guide: cv2.typing.MatLike, sigmaSpatial: float, sigmaColor: float, mode: int = ..., numIters: int = ...) -> DTFilter: ...
@_typing.overload
def createDTFilter(guide: cv2.UMat, sigmaSpatial: float, sigmaColor: float, mode: int = ..., numIters: int = ...) -> DTFilter: ...

def createDisparityWLSFilter(matcher_left: cv2.StereoMatcher) -> DisparityWLSFilter: ...

def createDisparityWLSFilterGeneric(use_confidence: bool) -> DisparityWLSFilter: ...

def createEdgeAwareInterpolator() -> EdgeAwareInterpolator: ...

def createEdgeBoxes(alpha: float = ..., beta: float = ..., eta: float = ..., minScore: float = ..., maxBoxes: int = ..., edgeMinMag: float = ..., edgeMergeThr: float = ..., clusterMinMag: float = ..., maxAspectRatio: float = ..., minBoxArea: float = ..., gamma: float = ..., kappa: float = ...) -> EdgeBoxes: ...

def createEdgeDrawing() -> EdgeDrawing: ...

@_typing.overload
def createFastBilateralSolverFilter(guide: cv2.typing.MatLike, sigma_spatial: float, sigma_luma: float, sigma_chroma: float, lambda_: float = ..., num_iter: int = ..., max_tol: float = ...) -> FastBilateralSolverFilter: ...
@_typing.overload
def createFastBilateralSolverFilter(guide: cv2.UMat, sigma_spatial: float, sigma_luma: float, sigma_chroma: float, lambda_: float = ..., num_iter: int = ..., max_tol: float = ...) -> FastBilateralSolverFilter: ...

@_typing.overload
def createFastGlobalSmootherFilter(guide: cv2.typing.MatLike, lambda_: float, sigma_color: float, lambda_attenuation: float = ..., num_iter: int = ...) -> FastGlobalSmootherFilter: ...
@_typing.overload
def createFastGlobalSmootherFilter(guide: cv2.UMat, lambda_: float, sigma_color: float, lambda_attenuation: float = ..., num_iter: int = ...) -> FastGlobalSmootherFilter: ...

def createFastLineDetector(length_threshold: int = ..., distance_threshold: float = ..., canny_th1: float = ..., canny_th2: float = ..., canny_aperture_size: int = ..., do_merge: bool = ...) -> FastLineDetector: ...

@_typing.overload
def createGuidedFilter(guide: cv2.typing.MatLike, radius: int, eps: float, scale: float = ...) -> GuidedFilter: ...
@_typing.overload
def createGuidedFilter(guide: cv2.UMat, radius: int, eps: float, scale: float = ...) -> GuidedFilter: ...

@_typing.overload
def createQuaternionImage(img: cv2.typing.MatLike, qimg: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
@_typing.overload
def createQuaternionImage(img: cv2.UMat, qimg: cv2.UMat | None = ...) -> cv2.UMat: ...

def createRFFeatureGetter() -> RFFeatureGetter: ...

def createRICInterpolator() -> RICInterpolator: ...

def createRightMatcher(matcher_left: cv2.StereoMatcher) -> cv2.StereoMatcher: ...

def createScanSegment(image_width: int, image_height: int, num_superpixels: int, slices: int = ..., merge_small: bool = ...) -> ScanSegment: ...

def createStructuredEdgeDetection(model: str, howToGetFeatures: RFFeatureGetter = ...) -> StructuredEdgeDetection: ...

@_typing.overload
def createSuperpixelLSC(image: cv2.typing.MatLike, region_size: int = ..., ratio: float = ...) -> SuperpixelLSC: ...
@_typing.overload
def createSuperpixelLSC(image: cv2.UMat, region_size: int = ..., ratio: float = ...) -> SuperpixelLSC: ...

def createSuperpixelSEEDS(image_width: int, image_height: int, image_channels: int, num_superpixels: int, num_levels: int, prior: int = ..., histogram_bins: int = ..., double_step: bool = ...) -> SuperpixelSEEDS: ...

@_typing.overload
def createSuperpixelSLIC(image: cv2.typing.MatLike, algorithm: int = ..., region_size: int = ..., ruler: float = ...) -> SuperpixelSLIC: ...
@_typing.overload
def createSuperpixelSLIC(image: cv2.UMat, algorithm: int = ..., region_size: int = ..., ruler: float = ...) -> SuperpixelSLIC: ...

@_typing.overload
def dtFilter(guide: cv2.typing.MatLike, src: cv2.typing.MatLike, sigmaSpatial: float, sigmaColor: float, dst: cv2.typing.MatLike | None = ..., mode: int = ..., numIters: int = ...) -> cv2.typing.MatLike: ...
@_typing.overload
def dtFilter(guide: cv2.UMat, src: cv2.UMat, sigmaSpatial: float, sigmaColor: float, dst: cv2.UMat | None = ..., mode: int = ..., numIters: int = ...) -> cv2.UMat: ...

@_typing.overload
def edgePreservingFilter(src: cv2.typing.MatLike, d: int, threshold: float, dst: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
@_typing.overload
def edgePreservingFilter(src: cv2.UMat, d: int, threshold: float, dst: cv2.UMat | None = ...) -> cv2.UMat: ...

@_typing.overload
def fastBilateralSolverFilter(guide: cv2.typing.MatLike, src: cv2.typing.MatLike, confidence: cv2.typing.MatLike, dst: cv2.typing.MatLike | None = ..., sigma_spatial: float = ..., sigma_luma: float = ..., sigma_chroma: float = ..., lambda_: float = ..., num_iter: int = ..., max_tol: float = ...) -> cv2.typing.MatLike: ...
@_typing.overload
def fastBilateralSolverFilter(guide: cv2.UMat, src: cv2.UMat, confidence: cv2.UMat, dst: cv2.UMat | None = ..., sigma_spatial: float = ..., sigma_luma: float = ..., sigma_chroma: float = ..., lambda_: float = ..., num_iter: int = ..., max_tol: float = ...) -> cv2.UMat: ...

@_typing.overload
def fastGlobalSmootherFilter(guide: cv2.typing.MatLike, src: cv2.typing.MatLike, lambda_: float, sigma_color: float, dst: cv2.typing.MatLike | None = ..., lambda_attenuation: float = ..., num_iter: int = ...) -> cv2.typing.MatLike: ...
@_typing.overload
def fastGlobalSmootherFilter(guide: cv2.UMat, src: cv2.UMat, lambda_: float, sigma_color: float, dst: cv2.UMat | None = ..., lambda_attenuation: float = ..., num_iter: int = ...) -> cv2.UMat: ...

@_typing.overload
def findEllipses(image: cv2.typing.MatLike, ellipses: cv2.typing.MatLike | None = ..., scoreThreshold: float = ..., reliabilityThreshold: float = ..., centerDistanceThreshold: float = ...) -> cv2.typing.MatLike: ...
@_typing.overload
def findEllipses(image: cv2.UMat, ellipses: cv2.UMat | None = ..., scoreThreshold: float = ..., reliabilityThreshold: float = ..., centerDistanceThreshold: float = ...) -> cv2.UMat: ...

@_typing.overload
def fourierDescriptor(src: cv2.typing.MatLike, dst: cv2.typing.MatLike | None = ..., nbElt: int = ..., nbFD: int = ...) -> cv2.typing.MatLike: ...
@_typing.overload
def fourierDescriptor(src: cv2.UMat, dst: cv2.UMat | None = ..., nbElt: int = ..., nbFD: int = ...) -> cv2.UMat: ...

@_typing.overload
def getDisparityVis(src: cv2.typing.MatLike, dst: cv2.typing.MatLike | None = ..., scale: float = ...) -> cv2.typing.MatLike: ...
@_typing.overload
def getDisparityVis(src: cv2.UMat, dst: cv2.UMat | None = ..., scale: float = ...) -> cv2.UMat: ...

@_typing.overload
def guidedFilter(guide: cv2.typing.MatLike, src: cv2.typing.MatLike, radius: int, eps: float, dst: cv2.typing.MatLike | None = ..., dDepth: int = ..., scale: float = ...) -> cv2.typing.MatLike: ...
@_typing.overload
def guidedFilter(guide: cv2.UMat, src: cv2.UMat, radius: int, eps: float, dst: cv2.UMat | None = ..., dDepth: int = ..., scale: float = ...) -> cv2.UMat: ...

@_typing.overload
def jointBilateralFilter(joint: cv2.typing.MatLike, src: cv2.typing.MatLike, d: int, sigmaColor: float, sigmaSpace: float, dst: cv2.typing.MatLike | None = ..., borderType: int = ...) -> cv2.typing.MatLike: ...
@_typing.overload
def jointBilateralFilter(joint: cv2.UMat, src: cv2.UMat, d: int, sigmaColor: float, sigmaSpace: float, dst: cv2.UMat | None = ..., borderType: int = ...) -> cv2.UMat: ...

@_typing.overload
def l0Smooth(src: cv2.typing.MatLike, dst: cv2.typing.MatLike | None = ..., lambda_: float = ..., kappa: float = ...) -> cv2.typing.MatLike: ...
@_typing.overload
def l0Smooth(src: cv2.UMat, dst: cv2.UMat | None = ..., lambda_: float = ..., kappa: float = ...) -> cv2.UMat: ...

@_typing.overload
def niBlackThreshold(_src: cv2.typing.MatLike, maxValue: float, type: int, blockSize: int, k: float, _dst: cv2.typing.MatLike | None = ..., binarizationMethod: int = ..., r: float = ...) -> cv2.typing.MatLike: ...
@_typing.overload
def niBlackThreshold(_src: cv2.UMat, maxValue: float, type: int, blockSize: int, k: float, _dst: cv2.UMat | None = ..., binarizationMethod: int = ..., r: float = ...) -> cv2.UMat: ...

@_typing.overload
def qconj(qimg: cv2.typing.MatLike, qcimg: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
@_typing.overload
def qconj(qimg: cv2.UMat, qcimg: cv2.UMat | None = ...) -> cv2.UMat: ...

@_typing.overload
def qdft(img: cv2.typing.MatLike, flags: int, sideLeft: bool, qimg: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
@_typing.overload
def qdft(img: cv2.UMat, flags: int, sideLeft: bool, qimg: cv2.UMat | None = ...) -> cv2.UMat: ...

@_typing.overload
def qmultiply(src1: cv2.typing.MatLike, src2: cv2.typing.MatLike, dst: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
@_typing.overload
def qmultiply(src1: cv2.UMat, src2: cv2.UMat, dst: cv2.UMat | None = ...) -> cv2.UMat: ...

@_typing.overload
def qunitary(qimg: cv2.typing.MatLike, qnimg: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
@_typing.overload
def qunitary(qimg: cv2.UMat, qnimg: cv2.UMat | None = ...) -> cv2.UMat: ...

@_typing.overload
def readGT(src_path: str, dst: cv2.typing.MatLike | None = ...) -> tuple[int, cv2.typing.MatLike]: ...
@_typing.overload
def readGT(src_path: str, dst: cv2.UMat | None = ...) -> tuple[int, cv2.UMat]: ...

@_typing.overload
def rollingGuidanceFilter(src: cv2.typing.MatLike, dst: cv2.typing.MatLike | None = ..., d: int = ..., sigmaColor: float = ..., sigmaSpace: float = ..., numOfIter: int = ..., borderType: int = ...) -> cv2.typing.MatLike: ...
@_typing.overload
def rollingGuidanceFilter(src: cv2.UMat, dst: cv2.UMat | None = ..., d: int = ..., sigmaColor: float = ..., sigmaSpace: float = ..., numOfIter: int = ..., borderType: int = ...) -> cv2.UMat: ...

@_typing.overload
def thinning(src: cv2.typing.MatLike, dst: cv2.typing.MatLike | None = ..., thinningType: int = ...) -> cv2.typing.MatLike: ...
@_typing.overload
def thinning(src: cv2.UMat, dst: cv2.UMat | None = ..., thinningType: int = ...) -> cv2.UMat: ...

@_typing.overload
def transformFD(src: cv2.typing.MatLike, t: cv2.typing.MatLike, dst: cv2.typing.MatLike | None = ..., fdContour: bool = ...) -> cv2.typing.MatLike: ...
@_typing.overload
def transformFD(src: cv2.UMat, t: cv2.UMat, dst: cv2.UMat | None = ..., fdContour: bool = ...) -> cv2.UMat: ...

@_typing.overload
def weightedMedianFilter(joint: cv2.typing.MatLike, src: cv2.typing.MatLike, r: int, dst: cv2.typing.MatLike | None = ..., sigma: float = ..., weightType: int = ..., mask: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
@_typing.overload
def weightedMedianFilter(joint: cv2.UMat, src: cv2.UMat, r: int, dst: cv2.UMat | None = ..., sigma: float = ..., weightType: int = ..., mask: cv2.UMat | None = ...) -> cv2.UMat: ...





__all__: list[str] = []

import cv2.typing
import os


# Enumerations
CV_SPECKLE_REMOVAL_ALGORITHM: int
CV_SPECKLE_REMOVAL_AVG_ALGORITHM: int
CV_QUADRATIC_INTERPOLATION: int
CV_SIMETRICV_INTERPOLATION: int
CV_DENSE_CENSUS: int
CV_SPARSE_CENSUS: int
CV_CS_CENSUS: int
CV_MODIFIED_CS_CENSUS: int
CV_MODIFIED_CENSUS_TRANSFORM: int
CV_MEAN_VARIATION: int
CV_STAR_KERNEL: int


StereoMatcher_DISP_SHIFT: int
STEREO_MATCHER_DISP_SHIFT: int
StereoMatcher_DISP_SCALE: int
STEREO_MATCHER_DISP_SCALE: int

StereoBinaryBM_PREFILTER_NORMALIZED_RESPONSE: int
STEREO_BINARY_BM_PREFILTER_NORMALIZED_RESPONSE: int
StereoBinaryBM_PREFILTER_XSOBEL: int
STEREO_BINARY_BM_PREFILTER_XSOBEL: int

StereoBinarySGBM_MODE_SGBM: int
STEREO_BINARY_SGBM_MODE_SGBM: int
StereoBinarySGBM_MODE_HH: int
STEREO_BINARY_SGBM_MODE_HH: int


# Classes
class MatchQuasiDense:
    p0: cv2.typing.Point2i
    p1: cv2.typing.Point2i
    corr: float

    # Functions
    def __init__(self) -> None: ...

    def apply(self, rhs: MatchQuasiDense) -> bool: ...


class PropagationParameters:
    corrWinSizeX: int
    corrWinSizeY: int
    borderX: int
    borderY: int
    correlationThreshold: float
    textrureThreshold: float
    neighborhoodSize: int
    disparityGradient: int
    lkTemplateSize: int
    lkPyrLvl: int
    lkTermParam1: int
    lkTermParam2: float
    gftQualityThres: float
    gftMinSeperationDist: int
    gftMaxNumFeatures: int

class QuasiDenseStereo:
    Param: PropagationParameters

    # Functions
    def loadParameters(self, filepath: str | os.PathLike[str]) -> int: ...

    def saveParameters(self, filepath: str | os.PathLike[str]) -> int: ...

    def getSparseMatches(self) -> _typing.Sequence[MatchQuasiDense]: ...

    def getDenseMatches(self) -> _typing.Sequence[MatchQuasiDense]: ...

    def process(self, imgLeft: cv2.typing.MatLike, imgRight: cv2.typing.MatLike) -> None: ...

    def getMatch(self, x: int, y: int) -> cv2.typing.Point2f: ...

    def getDisparity(self) -> cv2.typing.MatLike: ...

    @classmethod
    def create(cls, monoImgSize: cv2.typing.Size, paramFilepath: str = ...) -> QuasiDenseStereo: ...






import argparse
import glob
import os
import subprocess


def execute(cmd):
    popen = subprocess.Popen(cmd,
                             stdout=subprocess.PIPE,
                             stderr=subprocess.PIPE)
    for stdout_line in iter(popen.stdout.readline, ''):
        print(stdout_line.rstrip())
    for stderr_line in iter(popen.stderr.readline, ''):
        print(stderr_line.rstrip())
    popen.stdout.close()
    popen.stderr.close()
    return_code = popen.wait()
    if return_code != 0:
        raise subprocess.CalledProcessError(return_code, cmd)


def main():
    parser = argparse.ArgumentParser(
        description='Train Global Patch Collider using Middlebury dataset')
    parser.add_argument(
        '--bin_path',
        help='Path to the training executable (example_optflow_gpc_train)',
        required=True)
    parser.add_argument('--dataset_path',
                        help='Path to the directory with frames',
                        required=True)
    parser.add_argument('--gt_path',
                        help='Path to the directory with ground truth flow',
                        required=True)
    parser.add_argument('--descriptor_type',
                        help='Descriptor type',
                        type=int,
                        default=0)
    args = parser.parse_args()
    seq = glob.glob(os.path.join(args.dataset_path, '*'))
    seq.sort()
    input_files = []
    for s in seq:
        if os.path.isdir(s):
            seq_name = os.path.basename(s)
            frames = glob.glob(os.path.join(s, 'frame*.png'))
            frames.sort()
            assert (len(frames) == 2)
            assert (os.path.basename(frames[0]) == 'frame10.png')
            assert (os.path.basename(frames[1]) == 'frame11.png')
            gt_flow = os.path.join(args.gt_path, seq_name, 'flow10.flo')
            if os.path.isfile(gt_flow):
                input_files += [frames[0], frames[1], gt_flow]
    execute([args.bin_path, '--descriptor-type=%d' % args.descriptor_type] + input_files)


if __name__ == '__main__':
    main()



__all__: list[str] = []

import cv2.typing
import typing as _typing


# Classes
class CoreML:
    # Functions
    def __init__(self) -> None: ...

    def cfgUseCPUOnly(self) -> CoreML: ...

    def cfgEnableOnSubgraph(self) -> CoreML: ...

    def cfgEnableOnlyNeuralEngine(self) -> CoreML: ...


class CUDA:
    # Functions
    @_typing.overload
    def __init__(self) -> None: ...
    @_typing.overload
    def __init__(self, dev_id: int) -> None: ...


class TensorRT:
    # Functions
    @_typing.overload
    def __init__(self) -> None: ...
    @_typing.overload
    def __init__(self, dev_id: int) -> None: ...


class OpenVINO:
    # Functions
    @_typing.overload
    def __init__(self) -> None: ...
    @_typing.overload
    def __init__(self, dev_type: str) -> None: ...
    @_typing.overload
    def __init__(self, params: cv2.typing.map_string_and_string) -> None: ...

    def cfgCacheDir(self, dir: str) -> OpenVINO: ...

    def cfgNumThreads(self, nthreads: int) -> OpenVINO: ...

    def cfgEnableOpenCLThrottling(self) -> OpenVINO: ...

    def cfgEnableDynamicShapes(self) -> OpenVINO: ...


class DirectML:
    # Functions
    @_typing.overload
    def __init__(self) -> None: ...
    @_typing.overload
    def __init__(self, device_id: int) -> None: ...
    @_typing.overload
    def __init__(self, adapter_name: str) -> None: ...






__all__: list[str] = []

# Enumerations
AccelType_HOST: int
ACCEL_TYPE_HOST: int
AccelType_DX11: int
ACCEL_TYPE_DX11: int
AccelType_VAAPI: int
ACCEL_TYPE_VAAPI: int
AccelType_LAST_VALUE: int
ACCEL_TYPE_LAST_VALUE: int
AccelType = int
"""One of [AccelType_HOST, ACCEL_TYPE_HOST, AccelType_DX11, ACCEL_TYPE_DX11, AccelType_VAAPI, ACCEL_TYPE_VAAPI, AccelType_LAST_VALUE, ACCEL_TYPE_LAST_VALUE]"""






#!/usr/bin/python

import cv2 as cv
import numpy as np
import sys

img1 = cv.imread(sys.argv[1])
img1 = img1.astype(np.float32)

shift = np.array([5., 5.])

mapTest = cv.reg.MapShift(shift)
img2 = mapTest.warp(img1)
mapper = cv.reg.MapperGradShift()
mappPyr = cv.reg.MapperPyramid(mapper)

resMap = mappPyr.calculate(img1, img2)
mapShift = cv.reg.MapTypeCaster_toShift(resMap)

print(mapShift.getShift())



#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''
This sample demonstrates structured edge detection and edgeboxes.
Usage:
  edgeboxes_demo.py [<model>] [<input_image>]
'''

import cv2 as cv
import numpy as np
import sys

if __name__ == '__main__':
    print(__doc__)

    model = sys.argv[1]
    im = cv.imread(sys.argv[2])

    edge_detection = cv.ximgproc.createStructuredEdgeDetection(model)
    rgb_im = cv.cvtColor(im, cv.COLOR_BGR2RGB)
    edges = edge_detection.detectEdges(np.float32(rgb_im) / 255.0)

    orimap = edge_detection.computeOrientation(edges)
    edges = edge_detection.edgesNms(edges, orimap)

    edge_boxes = cv.ximgproc.createEdgeBoxes()
    edge_boxes.setMaxBoxes(30)
    boxes = edge_boxes.getBoundingBoxes(edges, orimap)
    boxes, scores = edge_boxes.getBoundingBoxes(edges, orimap)

    if len(boxes) > 0:
        boxes_scores = zip(boxes, scores)
        for b_s in boxes_scores:
            box = b_s[0]
            x, y, w, h = box
            cv.rectangle(im, (x, y), (x+w, y+h), (0, 255, 0), 1, cv.LINE_AA)
            score = b_s[1][0]
            cv.putText(im, "{:.2f}".format(score), (x, y), cv.FONT_HERSHEY_PLAIN, 0.8, (255, 255, 255), 1, cv.LINE_AA)
            print("Box at (x,y)=({:d},{:d}); score={:f}".format(x, y, score))

    cv.imshow("edges", edges)
    cv.imshow("edgeboxes", im)
    cv.waitKey(0)
    cv.destroyAllWindows()



import numpy as np
import cv2 as cv
import sys

from argparse import ArgumentParser

def get_depth_list(folder):
    f = open(folder + '/depth.txt', 'r')
    rgb = [folder + '/' + s for s in f.read().split() if s.endswith('.png')]
    return rgb

def kinfu_demo():
    parser = ArgumentParser()
    parser.add_argument(
        "-i", "--input", help="Required. Path to folder with a input image file", required=True, type=str)
    parser.add_argument(
        "-t", "--large_kinfu", help="Required. Name of KinFu type", required=False, type=str)
    parser.add_argument(
        "-ocl", "--use_opencl", help="Required. Flag of OpenCL use", required=False, type=int, default=1)

    args = parser.parse_args()
    print("Args: ", args)

    cv.ocl.setUseOpenCL(args.use_opencl)

    if (args.large_kinfu == None or args.large_kinfu == "0"):
        params = cv.kinfu_Params.defaultParams()
        kf = cv.kinfu_KinFu.create(params)
    elif (args.large_kinfu == "1"):
        params = cv.kinfu_Params.hashTSDFParams(False)
        kf = cv.kinfu_KinFu.create(params)
    else:
        raise ValueError("Incorrect kinfu type name")

    depth_list = get_depth_list(args.input)
    for path in depth_list:

        image = cv.imread(path, cv.IMREAD_ANYDEPTH)
        (height, width) = image.shape

        cv.imshow('input', image)

        size = height, width, 4
        cvt8 = np.zeros(size, dtype=np.uint8)

        if not kf.update(image):
            kf.reset()
        else:
            kf.render(cvt8)
            cv.imshow('render', cvt8)
        cv.pollKey()
    cv.waitKey(0)


if __name__ == '__main__':
    print(__doc__)
    kinfu_demo()
    cv.destroyAllWindows()



__all__: list[str] = []

import cv2
import cv2.typing
import typing as _typing


# Classes
class Plot2d(cv2.Algorithm):
    # Functions
    def setMinX(self, _plotMinX: float) -> None: ...

    def setMinY(self, _plotMinY: float) -> None: ...

    def setMaxX(self, _plotMaxX: float) -> None: ...

    def setMaxY(self, _plotMaxY: float) -> None: ...

    def setPlotLineWidth(self, _plotLineWidth: int) -> None: ...

    def setNeedPlotLine(self, _needPlotLine: bool) -> None: ...

    def setPlotLineColor(self, _plotLineColor: cv2.typing.Scalar) -> None: ...

    def setPlotBackgroundColor(self, _plotBackgroundColor: cv2.typing.Scalar) -> None: ...

    def setPlotAxisColor(self, _plotAxisColor: cv2.typing.Scalar) -> None: ...

    def setPlotGridColor(self, _plotGridColor: cv2.typing.Scalar) -> None: ...

    def setPlotTextColor(self, _plotTextColor: cv2.typing.Scalar) -> None: ...

    def setPlotSize(self, _plotSizeWidth: int, _plotSizeHeight: int) -> None: ...

    def setShowGrid(self, needShowGrid: bool) -> None: ...

    def setShowText(self, needShowText: bool) -> None: ...

    def setGridLinesNumber(self, gridLinesNumber: int) -> None: ...

    def setInvertOrientation(self, _invertOrientation: bool) -> None: ...

    def setPointIdxToPrint(self, pointIdx: int) -> None: ...

    @_typing.overload
    def render(self, _plotResult: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
    @_typing.overload
    def render(self, _plotResult: cv2.UMat | None = ...) -> cv2.UMat: ...

    @classmethod
    @_typing.overload
    def create(cls, data: cv2.typing.MatLike) -> Plot2d: ...
    @classmethod
    @_typing.overload
    def create(cls, data: cv2.UMat) -> Plot2d: ...
    @classmethod
    @_typing.overload
    def create(cls, dataX: cv2.typing.MatLike, dataY: cv2.typing.MatLike) -> Plot2d: ...
    @classmethod
    @_typing.overload
    def create(cls, dataX: cv2.UMat, dataY: cv2.UMat) -> Plot2d: ...






__all__: list[str] = []

import cv2
import cv2.typing
import typing as _typing


# Classes
class Tracker(cv2.Algorithm):
    # Functions
    @_typing.overload
    def init(self, image: cv2.typing.MatLike, boundingBox: cv2.typing.Rect2d) -> bool: ...
    @_typing.overload
    def init(self, image: cv2.UMat, boundingBox: cv2.typing.Rect2d) -> bool: ...

    @_typing.overload
    def update(self, image: cv2.typing.MatLike) -> tuple[bool, cv2.typing.Rect2d]: ...
    @_typing.overload
    def update(self, image: cv2.UMat) -> tuple[bool, cv2.typing.Rect2d]: ...


class TrackerMIL(Tracker):
    # Functions
    @classmethod
    def create(cls) -> TrackerMIL: ...


class TrackerBoosting(Tracker):
    # Functions
    @classmethod
    def create(cls) -> TrackerBoosting: ...


class TrackerMedianFlow(Tracker):
    # Functions
    @classmethod
    def create(cls) -> TrackerMedianFlow: ...


class TrackerTLD(Tracker):
    # Functions
    @classmethod
    def create(cls) -> TrackerTLD: ...


class TrackerKCF(Tracker):
    # Functions
    @classmethod
    def create(cls) -> TrackerKCF: ...


class TrackerMOSSE(Tracker):
    # Functions
    @classmethod
    def create(cls) -> TrackerMOSSE: ...


class MultiTracker(cv2.Algorithm):
    # Functions
    def __init__(self) -> None: ...

    @_typing.overload
    def add(self, newTracker: Tracker, image: cv2.typing.MatLike, boundingBox: cv2.typing.Rect2d) -> bool: ...
    @_typing.overload
    def add(self, newTracker: Tracker, image: cv2.UMat, boundingBox: cv2.typing.Rect2d) -> bool: ...

    @_typing.overload
    def update(self, image: cv2.typing.MatLike) -> tuple[bool, _typing.Sequence[cv2.typing.Rect2d]]: ...
    @_typing.overload
    def update(self, image: cv2.UMat) -> tuple[bool, _typing.Sequence[cv2.typing.Rect2d]]: ...

    def getObjects(self) -> _typing.Sequence[cv2.typing.Rect2d]: ...

    @classmethod
    def create(cls) -> MultiTracker: ...


class TrackerCSRT(Tracker):
    # Functions
    @classmethod
    def create(cls) -> TrackerCSRT: ...

    @_typing.overload
    def setInitialMask(self, mask: cv2.typing.MatLike) -> None: ...
    @_typing.overload
    def setInitialMask(self, mask: cv2.UMat) -> None: ...



# Functions
def upgradeTrackingAPI(legacy_tracker: Tracker) -> Tracker: ...





#!/usr/bin/python

import sys
import os

import cv2 as cv
import numpy as np

print('\ntextdetection.py')
print('       A demo script of the Extremal Region Filter algorithm described in:')
print('       Neumann L., Matas J.: Real-Time Scene Text Localization and Recognition, CVPR 2012\n')


if (len(sys.argv) < 2):
  print(' (ERROR) You must call this script with an argument (path_to_image_to_be_processed)\n')
  quit()

pathname = os.path.dirname(sys.argv[0])

img      = cv.imread(str(sys.argv[1]))
# for visualization
vis      = img.copy()


# Extract channels to be processed individually
channels = list(cv.text.computeNMChannels(img))
# Append negative channels to detect ER- (bright regions over dark background)
cn = len(channels)-1
for c in range(0,cn):
  channels.append(255-channels[c])

# Apply the default cascade classifier to each independent channel (could be done in parallel)

erc1 = cv.text.loadClassifierNM1('trained_classifierNM1.xml')
er1 = cv.text.createERFilterNM1(erc1,16,0.00015,0.13,0.2,True,0.1)

erc2 = cv.text.loadClassifierNM2('trained_classifierNM2.xml')
er2 = cv.text.createERFilterNM2(erc2,0.5)

print("Extracting Class Specific Extremal Regions from "+str(len(channels))+" channels ...")
print("    (...) this may take a while (...)")
for channel in channels:

  regions = cv.text.detectRegions(channel,er1,er2)

  rects = cv.text.erGrouping(img,channel,[r.tolist() for r in regions])
  #rects = cv.text.erGrouping(img,channel,[x.tolist() for x in regions], cv.text.ERGROUPING_ORIENTATION_ANY,'../../GSoC2014/opencv_contrib/modules/text/samples/trained_classifier_erGrouping.xml',0.5)

  #Visualization
  for rect in rects:
    cv.rectangle(vis, (rect[0],rect[1]), (rect[0]+rect[2],rect[1]+rect[3]), (0, 0, 0), 2)
    cv.rectangle(vis, (rect[0],rect[1]), (rect[0]+rect[2],rect[1]+rect[3]), (255, 255, 255), 1)

#Visualization
cv.imshow("Text detection result", vis)
cv.waitKey(0)



__all__: list[str] = []

# Enumerations
POINTS: int
LINES: int
LINE_LOOP: int
LINE_STRIP: int
TRIANGLES: int
TRIANGLE_STRIP: int
TRIANGLE_FAN: int
QUADS: int
QUAD_STRIP: int
POLYGON: int
RenderModes = int
"""One of [POINTS, LINES, LINE_LOOP, LINE_STRIP, TRIANGLES, TRIANGLE_STRIP, TRIANGLE_FAN, QUADS, QUAD_STRIP, POLYGON]"""


Buffer_ARRAY_BUFFER: int
BUFFER_ARRAY_BUFFER: int
Buffer_ELEMENT_ARRAY_BUFFER: int
BUFFER_ELEMENT_ARRAY_BUFFER: int
Buffer_PIXEL_PACK_BUFFER: int
BUFFER_PIXEL_PACK_BUFFER: int
Buffer_PIXEL_UNPACK_BUFFER: int
BUFFER_PIXEL_UNPACK_BUFFER: int
Buffer_Target = int
"""One of [Buffer_ARRAY_BUFFER, BUFFER_ARRAY_BUFFER, Buffer_ELEMENT_ARRAY_BUFFER, BUFFER_ELEMENT_ARRAY_BUFFER, Buffer_PIXEL_PACK_BUFFER, BUFFER_PIXEL_PACK_BUFFER, Buffer_PIXEL_UNPACK_BUFFER, BUFFER_PIXEL_UNPACK_BUFFER]"""

Buffer_READ_ONLY: int
BUFFER_READ_ONLY: int
Buffer_WRITE_ONLY: int
BUFFER_WRITE_ONLY: int
Buffer_READ_WRITE: int
BUFFER_READ_WRITE: int
Buffer_Access = int
"""One of [Buffer_READ_ONLY, BUFFER_READ_ONLY, Buffer_WRITE_ONLY, BUFFER_WRITE_ONLY, Buffer_READ_WRITE, BUFFER_READ_WRITE]"""

Texture2D_NONE: int
TEXTURE2D_NONE: int
Texture2D_DEPTH_COMPONENT: int
TEXTURE2D_DEPTH_COMPONENT: int
Texture2D_RGB: int
TEXTURE2D_RGB: int
Texture2D_RGBA: int
TEXTURE2D_RGBA: int
Texture2D_Format = int
"""One of [Texture2D_NONE, TEXTURE2D_NONE, Texture2D_DEPTH_COMPONENT, TEXTURE2D_DEPTH_COMPONENT, Texture2D_RGB, TEXTURE2D_RGB, Texture2D_RGBA, TEXTURE2D_RGBA]"""


# Classes




__all__: list[str] = []

import cv2
import cv2.typing
import typing as _typing


from cv2.gapi import core as core
from cv2.gapi import ie as ie
from cv2.gapi import imgproc as imgproc
from cv2.gapi import oak as oak
from cv2.gapi import onnx as onnx
from cv2.gapi import ot as ot
from cv2.gapi import ov as ov
from cv2.gapi import own as own
from cv2.gapi import render as render
from cv2.gapi import streaming as streaming
from cv2.gapi import video as video
from cv2.gapi import wip as wip


# Enumerations
StereoOutputFormat_DEPTH_FLOAT16: int
STEREO_OUTPUT_FORMAT_DEPTH_FLOAT16: int
StereoOutputFormat_DEPTH_FLOAT32: int
STEREO_OUTPUT_FORMAT_DEPTH_FLOAT32: int
StereoOutputFormat_DISPARITY_FIXED16_11_5: int
STEREO_OUTPUT_FORMAT_DISPARITY_FIXED16_11_5: int
StereoOutputFormat_DISPARITY_FIXED16_12_4: int
STEREO_OUTPUT_FORMAT_DISPARITY_FIXED16_12_4: int
StereoOutputFormat_DEPTH_16F: int
STEREO_OUTPUT_FORMAT_DEPTH_16F: int
StereoOutputFormat_DEPTH_32F: int
STEREO_OUTPUT_FORMAT_DEPTH_32F: int
StereoOutputFormat_DISPARITY_16Q_10_5: int
STEREO_OUTPUT_FORMAT_DISPARITY_16Q_10_5: int
StereoOutputFormat_DISPARITY_16Q_11_4: int
STEREO_OUTPUT_FORMAT_DISPARITY_16Q_11_4: int
StereoOutputFormat = int
"""One of [StereoOutputFormat_DEPTH_FLOAT16, STEREO_OUTPUT_FORMAT_DEPTH_FLOAT16, StereoOutputFormat_DEPTH_FLOAT32, STEREO_OUTPUT_FORMAT_DEPTH_FLOAT32, StereoOutputFormat_DISPARITY_FIXED16_11_5, STEREO_OUTPUT_FORMAT_DISPARITY_FIXED16_11_5, StereoOutputFormat_DISPARITY_FIXED16_12_4, STEREO_OUTPUT_FORMAT_DISPARITY_FIXED16_12_4, StereoOutputFormat_DEPTH_16F, STEREO_OUTPUT_FORMAT_DEPTH_16F, StereoOutputFormat_DEPTH_32F, STEREO_OUTPUT_FORMAT_DEPTH_32F, StereoOutputFormat_DISPARITY_16Q_10_5, STEREO_OUTPUT_FORMAT_DISPARITY_16Q_10_5, StereoOutputFormat_DISPARITY_16Q_11_4, STEREO_OUTPUT_FORMAT_DISPARITY_16Q_11_4]"""

CV_BOOL: int
CV_INT: int
CV_INT64: int
CV_UINT64: int
CV_DOUBLE: int
CV_FLOAT: int
CV_STRING: int
CV_POINT: int
CV_POINT2F: int
CV_POINT3F: int
CV_SIZE: int
CV_RECT: int
CV_SCALAR: int
CV_MAT: int
CV_GMAT: int
CV_DRAW_PRIM: int
CV_ANY: int
ArgType = int
"""One of [CV_BOOL, CV_INT, CV_INT64, CV_UINT64, CV_DOUBLE, CV_FLOAT, CV_STRING, CV_POINT, CV_POINT2F, CV_POINT3F, CV_SIZE, CV_RECT, CV_SCALAR, CV_MAT, CV_GMAT, CV_DRAW_PRIM, CV_ANY]"""



# Classes
class GNetParam:
    ...

class GNetPackage:
    # Functions
    @_typing.overload
    def __init__(self) -> None: ...
    @_typing.overload
    def __init__(self, nets: _typing.Sequence[GNetParam]) -> None: ...



# Functions
def BGR2Gray(src: cv2.GMat) -> cv2.GMat: ...

def BGR2I420(src: cv2.GMat) -> cv2.GMat: ...

def BGR2LUV(src: cv2.GMat) -> cv2.GMat: ...

def BGR2RGB(src: cv2.GMat) -> cv2.GMat: ...

def BGR2YUV(src: cv2.GMat) -> cv2.GMat: ...

def BayerGR2RGB(src_gr: cv2.GMat) -> cv2.GMat: ...

def Canny(image: cv2.GMat, threshold1: float, threshold2: float, apertureSize: int = ..., L2gradient: bool = ...) -> cv2.GMat: ...

def I4202BGR(src: cv2.GMat) -> cv2.GMat: ...

def I4202RGB(src: cv2.GMat) -> cv2.GMat: ...

def LUT(src: cv2.GMat, lut: cv2.typing.MatLike) -> cv2.GMat: ...

def LUV2BGR(src: cv2.GMat) -> cv2.GMat: ...

def Laplacian(src: cv2.GMat, ddepth: int, ksize: int = ..., scale: float = ..., delta: float = ..., borderType: int = ...) -> cv2.GMat: ...

def NV12toBGR(src_y: cv2.GMat, src_uv: cv2.GMat) -> cv2.GMat: ...

def NV12toGray(src_y: cv2.GMat, src_uv: cv2.GMat) -> cv2.GMat: ...

def NV12toRGB(src_y: cv2.GMat, src_uv: cv2.GMat) -> cv2.GMat: ...

@_typing.overload
def RGB2Gray(src: cv2.GMat) -> cv2.GMat: ...
@_typing.overload
def RGB2Gray(src: cv2.GMat, rY: float, gY: float, bY: float) -> cv2.GMat: ...

def RGB2HSV(src: cv2.GMat) -> cv2.GMat: ...

def RGB2I420(src: cv2.GMat) -> cv2.GMat: ...

def RGB2Lab(src: cv2.GMat) -> cv2.GMat: ...

def RGB2YUV(src: cv2.GMat) -> cv2.GMat: ...

def RGB2YUV422(src: cv2.GMat) -> cv2.GMat: ...

def Sobel(src: cv2.GMat, ddepth: int, dx: int, dy: int, ksize: int = ..., scale: float = ..., delta: float = ..., borderType: int = ..., borderValue: cv2.typing.Scalar = ...) -> cv2.GMat: ...

def SobelXY(src: cv2.GMat, ddepth: int, order: int, ksize: int = ..., scale: float = ..., delta: float = ..., borderType: int = ..., borderValue: cv2.typing.Scalar = ...) -> tuple[cv2.GMat, cv2.GMat]: ...

def YUV2BGR(src: cv2.GMat) -> cv2.GMat: ...

def YUV2RGB(src: cv2.GMat) -> cv2.GMat: ...

def absDiff(src1: cv2.GMat, src2: cv2.GMat) -> cv2.GMat: ...

def absDiffC(src: cv2.GMat, c: cv2.GScalar) -> cv2.GMat: ...

def add(src1: cv2.GMat, src2: cv2.GMat, ddepth: int = ...) -> cv2.GMat: ...

@_typing.overload
def addC(src1: cv2.GMat, c: cv2.GScalar, ddepth: int = ...) -> cv2.GMat: ...
@_typing.overload
def addC(c: cv2.GScalar, src1: cv2.GMat, ddepth: int = ...) -> cv2.GMat: ...

def addWeighted(src1: cv2.GMat, alpha: float, src2: cv2.GMat, beta: float, gamma: float, ddepth: int = ...) -> cv2.GMat: ...

def bilateralFilter(src: cv2.GMat, d: int, sigmaColor: float, sigmaSpace: float, borderType: int = ...) -> cv2.GMat: ...

@_typing.overload
def bitwise_and(src1: cv2.GMat, src2: cv2.GMat) -> cv2.GMat: ...
@_typing.overload
def bitwise_and(src1: cv2.GMat, src2: cv2.GScalar) -> cv2.GMat: ...

def bitwise_not(src: cv2.GMat) -> cv2.GMat: ...

@_typing.overload
def bitwise_or(src1: cv2.GMat, src2: cv2.GMat) -> cv2.GMat: ...
@_typing.overload
def bitwise_or(src1: cv2.GMat, src2: cv2.GScalar) -> cv2.GMat: ...

@_typing.overload
def bitwise_xor(src1: cv2.GMat, src2: cv2.GMat) -> cv2.GMat: ...
@_typing.overload
def bitwise_xor(src1: cv2.GMat, src2: cv2.GScalar) -> cv2.GMat: ...

def blur(src: cv2.GMat, ksize: cv2.typing.Size, anchor: cv2.typing.Point = ..., borderType: int = ..., borderValue: cv2.typing.Scalar = ...) -> cv2.GMat: ...

@_typing.overload
def boundingRect(src: cv2.GMat) -> cv2.GOpaqueT: ...
@_typing.overload
def boundingRect(src: cv2.GArrayT) -> cv2.GOpaqueT: ...
@_typing.overload
def boundingRect(src: cv2.GArrayT) -> cv2.GOpaqueT: ...

def boxFilter(src: cv2.GMat, dtype: int, ksize: cv2.typing.Size, anchor: cv2.typing.Point = ..., normalize: bool = ..., borderType: int = ..., borderValue: cv2.typing.Scalar = ...) -> cv2.GMat: ...

def cartToPolar(x: cv2.GMat, y: cv2.GMat, angleInDegrees: bool = ...) -> tuple[cv2.GMat, cv2.GMat]: ...

@_typing.overload
def cmpEQ(src1: cv2.GMat, src2: cv2.GMat) -> cv2.GMat: ...
@_typing.overload
def cmpEQ(src1: cv2.GMat, src2: cv2.GScalar) -> cv2.GMat: ...

@_typing.overload
def cmpGE(src1: cv2.GMat, src2: cv2.GMat) -> cv2.GMat: ...
@_typing.overload
def cmpGE(src1: cv2.GMat, src2: cv2.GScalar) -> cv2.GMat: ...

@_typing.overload
def cmpGT(src1: cv2.GMat, src2: cv2.GMat) -> cv2.GMat: ...
@_typing.overload
def cmpGT(src1: cv2.GMat, src2: cv2.GScalar) -> cv2.GMat: ...

@_typing.overload
def cmpLE(src1: cv2.GMat, src2: cv2.GMat) -> cv2.GMat: ...
@_typing.overload
def cmpLE(src1: cv2.GMat, src2: cv2.GScalar) -> cv2.GMat: ...

@_typing.overload
def cmpLT(src1: cv2.GMat, src2: cv2.GMat) -> cv2.GMat: ...
@_typing.overload
def cmpLT(src1: cv2.GMat, src2: cv2.GScalar) -> cv2.GMat: ...

@_typing.overload
def cmpNE(src1: cv2.GMat, src2: cv2.GMat) -> cv2.GMat: ...
@_typing.overload
def cmpNE(src1: cv2.GMat, src2: cv2.GScalar) -> cv2.GMat: ...

def combine(lhs: cv2.GKernelPackage, rhs: cv2.GKernelPackage) -> cv2.GKernelPackage: ...

@_typing.overload
def concatHor(src1: cv2.GMat, src2: cv2.GMat) -> cv2.GMat: ...
@_typing.overload
def concatHor(v: _typing.Sequence[cv2.GMat]) -> cv2.GMat: ...

@_typing.overload
def concatVert(src1: cv2.GMat, src2: cv2.GMat) -> cv2.GMat: ...
@_typing.overload
def concatVert(v: _typing.Sequence[cv2.GMat]) -> cv2.GMat: ...

def convertTo(src: cv2.GMat, rdepth: int, alpha: float = ..., beta: float = ...) -> cv2.GMat: ...

def copy(in_: cv2.GMat) -> cv2.GMat: ...

def countNonZero(src: cv2.GMat) -> cv2.GOpaqueT: ...

def crop(src: cv2.GMat, rect: cv2.typing.Rect) -> cv2.GMat: ...

def dilate(src: cv2.GMat, kernel: cv2.typing.MatLike, anchor: cv2.typing.Point = ..., iterations: int = ..., borderType: int = ..., borderValue: cv2.typing.Scalar = ...) -> cv2.GMat: ...

def dilate3x3(src: cv2.GMat, iterations: int = ..., borderType: int = ..., borderValue: cv2.typing.Scalar = ...) -> cv2.GMat: ...

def div(src1: cv2.GMat, src2: cv2.GMat, scale: float, ddepth: int = ...) -> cv2.GMat: ...

def divC(src: cv2.GMat, divisor: cv2.GScalar, scale: float, ddepth: int = ...) -> cv2.GMat: ...

def divRC(divident: cv2.GScalar, src: cv2.GMat, scale: float, ddepth: int = ...) -> cv2.GMat: ...

def equalizeHist(src: cv2.GMat) -> cv2.GMat: ...

def erode(src: cv2.GMat, kernel: cv2.typing.MatLike, anchor: cv2.typing.Point = ..., iterations: int = ..., borderType: int = ..., borderValue: cv2.typing.Scalar = ...) -> cv2.GMat: ...

def erode3x3(src: cv2.GMat, iterations: int = ..., borderType: int = ..., borderValue: cv2.typing.Scalar = ...) -> cv2.GMat: ...

def filter2D(src: cv2.GMat, ddepth: int, kernel: cv2.typing.MatLike, anchor: cv2.typing.Point = ..., delta: cv2.typing.Scalar = ..., borderType: int = ..., borderValue: cv2.typing.Scalar = ...) -> cv2.GMat: ...

def flip(src: cv2.GMat, flipCode: int) -> cv2.GMat: ...

def gaussianBlur(src: cv2.GMat, ksize: cv2.typing.Size, sigmaX: float, sigmaY: float = ..., borderType: int = ..., borderValue: cv2.typing.Scalar = ...) -> cv2.GMat: ...

def goodFeaturesToTrack(image: cv2.GMat, maxCorners: int, qualityLevel: float, minDistance: float, mask: cv2.typing.MatLike | None = ..., blockSize: int = ..., useHarrisDetector: bool = ..., k: float = ...) -> cv2.GArrayT: ...

def inRange(src: cv2.GMat, threshLow: cv2.GScalar, threshUp: cv2.GScalar) -> cv2.GMat: ...

@_typing.overload
def infer(name: str, inputs: cv2.GInferInputs) -> cv2.GInferOutputs: ...
@_typing.overload
def infer(name: str, roi: cv2.GOpaqueT, inputs: cv2.GInferInputs) -> cv2.GInferOutputs: ...
@_typing.overload
def infer(name: str, rois: cv2.GArrayT, inputs: cv2.GInferInputs) -> cv2.GInferListOutputs: ...

def infer2(name: str, in_: cv2.GMat, inputs: cv2.GInferListInputs) -> cv2.GInferListOutputs: ...

def integral(src: cv2.GMat, sdepth: int = ..., sqdepth: int = ...) -> tuple[cv2.GMat, cv2.GMat]: ...

@_typing.overload
def kmeans(data: cv2.GMat, K: int, bestLabels: cv2.GMat, criteria: cv2.typing.TermCriteria, attempts: int, flags: cv2.KmeansFlags) -> tuple[cv2.GOpaqueT, cv2.GMat, cv2.GMat]: ...
@_typing.overload
def kmeans(data: cv2.GMat, K: int, criteria: cv2.typing.TermCriteria, attempts: int, flags: cv2.KmeansFlags) -> tuple[cv2.GOpaqueT, cv2.GMat, cv2.GMat]: ...
@_typing.overload
def kmeans(data: cv2.GArrayT, K: int, bestLabels: cv2.GArrayT, criteria: cv2.typing.TermCriteria, attempts: int, flags: cv2.KmeansFlags) -> tuple[cv2.GOpaqueT, cv2.GArrayT, cv2.GArrayT]: ...
@_typing.overload
def kmeans(data: cv2.GArrayT, K: int, bestLabels: cv2.GArrayT, criteria: cv2.typing.TermCriteria, attempts: int, flags: cv2.KmeansFlags) -> tuple[cv2.GOpaqueT, cv2.GArrayT, cv2.GArrayT]: ...

def mask(src: cv2.GMat, mask: cv2.GMat) -> cv2.GMat: ...

def max(src1: cv2.GMat, src2: cv2.GMat) -> cv2.GMat: ...

def mean(src: cv2.GMat) -> cv2.GScalar: ...

def medianBlur(src: cv2.GMat, ksize: int) -> cv2.GMat: ...

def merge3(src1: cv2.GMat, src2: cv2.GMat, src3: cv2.GMat) -> cv2.GMat: ...

def merge4(src1: cv2.GMat, src2: cv2.GMat, src3: cv2.GMat, src4: cv2.GMat) -> cv2.GMat: ...

def min(src1: cv2.GMat, src2: cv2.GMat) -> cv2.GMat: ...

def morphologyEx(src: cv2.GMat, op: cv2.MorphTypes, kernel: cv2.typing.MatLike, anchor: cv2.typing.Point = ..., iterations: int = ..., borderType: cv2.BorderTypes = ..., borderValue: cv2.typing.Scalar = ...) -> cv2.GMat: ...

def mul(src1: cv2.GMat, src2: cv2.GMat, scale: float = ..., ddepth: int = ...) -> cv2.GMat: ...

@_typing.overload
def mulC(src: cv2.GMat, multiplier: float, ddepth: int = ...) -> cv2.GMat: ...
@_typing.overload
def mulC(src: cv2.GMat, multiplier: cv2.GScalar, ddepth: int = ...) -> cv2.GMat: ...
@_typing.overload
def mulC(multiplier: cv2.GScalar, src: cv2.GMat, ddepth: int = ...) -> cv2.GMat: ...

def normInf(src: cv2.GMat) -> cv2.GScalar: ...

def normL1(src: cv2.GMat) -> cv2.GScalar: ...

def normL2(src: cv2.GMat) -> cv2.GScalar: ...

def normalize(src: cv2.GMat, alpha: float, beta: float, norm_type: int, ddepth: int = ...) -> cv2.GMat: ...

@_typing.overload
def parseSSD(in_: cv2.GMat, inSz: cv2.GOpaqueT, confidenceThreshold: float = ..., filterLabel: int = ...) -> tuple[cv2.GArrayT, cv2.GArrayT]: ...
@_typing.overload
def parseSSD(in_: cv2.GMat, inSz: cv2.GOpaqueT, confidenceThreshold: float, alignmentToSquare: bool, filterOutOfBounds: bool) -> cv2.GArrayT: ...

def parseYolo(in_: cv2.GMat, inSz: cv2.GOpaqueT, confidenceThreshold: float = ..., nmsThreshold: float = ..., anchors: _typing.Sequence[float] = ...) -> tuple[cv2.GArrayT, cv2.GArrayT]: ...

def phase(x: cv2.GMat, y: cv2.GMat, angleInDegrees: bool = ...) -> cv2.GMat: ...

def polarToCart(magnitude: cv2.GMat, angle: cv2.GMat, angleInDegrees: bool = ...) -> tuple[cv2.GMat, cv2.GMat]: ...

def remap(src: cv2.GMat, map1: cv2.typing.MatLike, map2: cv2.typing.MatLike, interpolation: int, borderMode: int = ..., borderValue: cv2.typing.Scalar = ...) -> cv2.GMat: ...

def resize(src: cv2.GMat, dsize: cv2.typing.Size, fx: float = ..., fy: float = ..., interpolation: int = ...) -> cv2.GMat: ...

def select(src1: cv2.GMat, src2: cv2.GMat, mask: cv2.GMat) -> cv2.GMat: ...

def sepFilter(src: cv2.GMat, ddepth: int, kernelX: cv2.typing.MatLike, kernelY: cv2.typing.MatLike, anchor: cv2.typing.Point, delta: cv2.typing.Scalar, borderType: int = ..., borderValue: cv2.typing.Scalar = ...) -> cv2.GMat: ...

def split3(src: cv2.GMat) -> tuple[cv2.GMat, cv2.GMat, cv2.GMat]: ...

def split4(src: cv2.GMat) -> tuple[cv2.GMat, cv2.GMat, cv2.GMat, cv2.GMat]: ...

def sqrt(src: cv2.GMat) -> cv2.GMat: ...

def sub(src1: cv2.GMat, src2: cv2.GMat, ddepth: int = ...) -> cv2.GMat: ...

def subC(src: cv2.GMat, c: cv2.GScalar, ddepth: int = ...) -> cv2.GMat: ...

def subRC(c: cv2.GScalar, src: cv2.GMat, ddepth: int = ...) -> cv2.GMat: ...

def sum(src: cv2.GMat) -> cv2.GScalar: ...

@_typing.overload
def threshold(src: cv2.GMat, thresh: cv2.GScalar, maxval: cv2.GScalar, type: int) -> cv2.GMat: ...
@_typing.overload
def threshold(src: cv2.GMat, maxval: cv2.GScalar, type: int) -> tuple[cv2.GMat, cv2.GScalar]: ...

def transpose(src: cv2.GMat) -> cv2.GMat: ...

def warpAffine(src: cv2.GMat, M: cv2.typing.MatLike, dsize: cv2.typing.Size, flags: int = ..., borderMode: int = ..., borderValue: cv2.typing.Scalar = ...) -> cv2.GMat: ...

def warpPerspective(src: cv2.GMat, M: cv2.typing.MatLike, dsize: cv2.typing.Size, flags: int = ..., borderMode: int = ..., borderValue: cv2.typing.Scalar = ...) -> cv2.GMat: ...





__all__: list[str] = []

import cv2
import cv2.typing
import typing as _typing


# Classes
class GraphSegmentation(cv2.Algorithm):
    # Functions
    @_typing.overload
    def processImage(self, src: cv2.typing.MatLike, dst: cv2.typing.MatLike | None = ...) -> cv2.typing.MatLike: ...
    @_typing.overload
    def processImage(self, src: cv2.UMat, dst: cv2.UMat | None = ...) -> cv2.UMat: ...

    def setSigma(self, sigma: float) -> None: ...

    def getSigma(self) -> float: ...

    def setK(self, k: float) -> None: ...

    def getK(self) -> float: ...

    def setMinSize(self, min_size: int) -> None: ...

    def getMinSize(self) -> int: ...


class SelectiveSearchSegmentationStrategy(cv2.Algorithm):
    # Functions
    @_typing.overload
    def setImage(self, img: cv2.typing.MatLike, regions: cv2.typing.MatLike, sizes: cv2.typing.MatLike, image_id: int = ...) -> None: ...
    @_typing.overload
    def setImage(self, img: cv2.UMat, regions: cv2.UMat, sizes: cv2.UMat, image_id: int = ...) -> None: ...

    def get(self, r1: int, r2: int) -> float: ...

    def merge(self, r1: int, r2: int) -> None: ...


class SelectiveSearchSegmentationStrategyColor(SelectiveSearchSegmentationStrategy):
    ...

class SelectiveSearchSegmentationStrategySize(SelectiveSearchSegmentationStrategy):
    ...

class SelectiveSearchSegmentationStrategyTexture(SelectiveSearchSegmentationStrategy):
    ...

class SelectiveSearchSegmentationStrategyFill(SelectiveSearchSegmentationStrategy):
    ...

class SelectiveSearchSegmentationStrategyMultiple(SelectiveSearchSegmentationStrategy):
    # Functions
    def addStrategy(self, g: SelectiveSearchSegmentationStrategy, weight: float) -> None: ...

    def clearStrategies(self) -> None: ...


class SelectiveSearchSegmentation(cv2.Algorithm):
    # Functions
    @_typing.overload
    def setBaseImage(self, img: cv2.typing.MatLike) -> None: ...
    @_typing.overload
    def setBaseImage(self, img: cv2.UMat) -> None: ...

    def switchToSingleStrategy(self, k: int = ..., sigma: float = ...) -> None: ...

    def switchToSelectiveSearchFast(self, base_k: int = ..., inc_k: int = ..., sigma: float = ...) -> None: ...

    def switchToSelectiveSearchQuality(self, base_k: int = ..., inc_k: int = ..., sigma: float = ...) -> None: ...

    @_typing.overload
    def addImage(self, img: cv2.typing.MatLike) -> None: ...
    @_typing.overload
    def addImage(self, img: cv2.UMat) -> None: ...

    def clearImages(self) -> None: ...

    def addGraphSegmentation(self, g: GraphSegmentation) -> None: ...

    def clearGraphSegmentations(self) -> None: ...

    def addStrategy(self, s: SelectiveSearchSegmentationStrategy) -> None: ...

    def clearStrategies(self) -> None: ...

    def process(self) -> _typing.Sequence[cv2.typing.Rect]: ...



# Functions
def createGraphSegmentation(sigma: float = ..., k: float = ..., min_size: int = ...) -> GraphSegmentation: ...

def createSelectiveSearchSegmentation() -> SelectiveSearchSegmentation: ...

def createSelectiveSearchSegmentationStrategyColor() -> SelectiveSearchSegmentationStrategyColor: ...

def createSelectiveSearchSegmentationStrategyFill() -> SelectiveSearchSegmentationStrategyFill: ...

@_typing.overload
def createSelectiveSearchSegmentationStrategyMultiple() -> SelectiveSearchSegmentationStrategyMultiple: ...
@_typing.overload
def createSelectiveSearchSegmentationStrategyMultiple(s1: SelectiveSearchSegmentationStrategy) -> SelectiveSearchSegmentationStrategyMultiple: ...
@_typing.overload
def createSelectiveSearchSegmentationStrategyMultiple(s1: SelectiveSearchSegmentationStrategy, s2: SelectiveSearchSegmentationStrategy) -> SelectiveSearchSegmentationStrategyMultiple: ...
@_typing.overload
def createSelectiveSearchSegmentationStrategyMultiple(s1: SelectiveSearchSegmentationStrategy, s2: SelectiveSearchSegmentationStrategy, s3: SelectiveSearchSegmentationStrategy) -> SelectiveSearchSegmentationStrategyMultiple: ...
@_typing.overload
def createSelectiveSearchSegmentationStrategyMultiple(s1: SelectiveSearchSegmentationStrategy, s2: SelectiveSearchSegmentationStrategy, s3: SelectiveSearchSegmentationStrategy, s4: SelectiveSearchSegmentationStrategy) -> SelectiveSearchSegmentationStrategyMultiple: ...

def createSelectiveSearchSegmentationStrategySize() -> SelectiveSearchSegmentationStrategySize: ...

def createSelectiveSearchSegmentationStrategyTexture() -> SelectiveSearchSegmentationStrategyTexture: ...





import numpy as np
import cv2 as cv
import math
import argparse

class AudioDrawing:
    '''
        Used for drawing audio graphics
    '''
    def __init__(self, args):

        self.inputType = args.inputType
        self.draw = args.draw
        self.graph = args.graph
        self.audio = cv.samples.findFile(args.audio)
        self.audioStream = args.audioStream

        self.windowType = args.windowType
        self.windLen = args.windLen
        self.overlap = args.overlap

        self.enableGrid = args.enableGrid

        self.rows = args.rows
        self.cols = args.cols

        self.xmarkup = args.xmarkup
        self.ymarkup = args.ymarkup
        self.zmarkup = args.zmarkup

        self.microTime = args.microTime
        self.frameSizeTime = args.frameSizeTime
        self.updateTime = args.updateTime
        self.waitTime = args.waitTime

        if self.initAndCheckArgs(args) is False:
            exit()


    def Draw(self):
        if self.draw == "static":

            if self.inputType == "file":
                samplingRate, inputAudio = self.readAudioFile(self.audio)

            elif self.inputType == "microphone":
                samplingRate, inputAudio = self.readAudioMicrophone()

            duration = len(inputAudio) // samplingRate

            # since the dimensional grid is counted in integer seconds,
            # if the input audio has an incomplete last second,
            # then it is filled with zeros to complete
            remainder = len(inputAudio) % samplingRate
            if remainder != 0:
                sizeToFullSec = samplingRate - remainder
                zeroArr = np.zeros(sizeToFullSec)
                inputAudio = np.concatenate((inputAudio, zeroArr), axis=0)
                duration += 1
                print("Update duration of audio to full second with ",
                    sizeToFullSec, " zero samples")
                print("New number of samples ", len(inputAudio))

            if duration <= self.xmarkup:
                self.xmarkup = duration + 1

            if self.graph == "ampl":
                imgAmplitude = self.drawAmplitude(inputAudio)
                imgAmplitude = self.drawAmplitudeScale(imgAmplitude, inputAudio, samplingRate)
                cv.imshow("Display window", imgAmplitude)
                cv.waitKey(0)

            elif self.graph == "spec":
                stft = self.STFT(inputAudio)
                imgSpec = self.drawSpectrogram(stft)
                imgSpec = self.drawSpectrogramColorbar(imgSpec, inputAudio, samplingRate, stft)
                cv.imshow("Display window", imgSpec)
                cv.waitKey(0)

            elif self.graph == "ampl_and_spec":
                imgAmplitude = self.drawAmplitude(inputAudio)
                imgAmplitude = self.drawAmplitudeScale(imgAmplitude, inputAudio, samplingRate)

                stft = self.STFT(inputAudio)
                imgSpec = self.drawSpectrogram(stft)
                imgSpec = self.drawSpectrogramColorbar(imgSpec, inputAudio, samplingRate, stft)

                imgTotal = self.concatenateImages(imgAmplitude, imgSpec)
                cv.imshow("Display window", imgTotal)
                cv.waitKey(0)

        elif self.draw == "dynamic":

            if self.inputType == "file":
                self.dynamicFile(self.audio)

            elif self.inputType == "microphone":
                self.dynamicMicrophone()


    def readAudioFile(self, file):
        cap = cv.VideoCapture(file)

        params = [cv.CAP_PROP_AUDIO_STREAM, self.audioStream,
                cv.CAP_PROP_VIDEO_STREAM, -1,
                cv.CAP_PROP_AUDIO_DATA_DEPTH, cv.CV_16S]
        params = np.asarray(params)

        cap.open(file, cv.CAP_ANY, params)
        if cap.isOpened() == False:
            print("Error : Can't read audio file: '", self.audio, "' with audioStream = ", self.audioStream)
            print("Error: problems with audio reading, check input arguments")
            exit()
        audioBaseIndex = int(cap.get(cv.CAP_PROP_AUDIO_BASE_INDEX))
        numberOfChannels = int(cap.get(cv.CAP_PROP_AUDIO_TOTAL_CHANNELS))

        print("CAP_PROP_AUDIO_DATA_DEPTH: ", str((int(cap.get(cv.CAP_PROP_AUDIO_DATA_DEPTH)))))
        print("CAP_PROP_AUDIO_SAMPLES_PER_SECOND: ", cap.get(cv.CAP_PROP_AUDIO_SAMPLES_PER_SECOND))
        print("CAP_PROP_AUDIO_TOTAL_CHANNELS: ", numberOfChannels)
        print("CAP_PROP_AUDIO_TOTAL_STREAMS: ", cap.get(cv.CAP_PROP_AUDIO_TOTAL_STREAMS))

        frame = []
        frame = np.asarray(frame)
        inputAudio = []

        while (1):
            if (cap.grab()):
                frame = []
                frame = np.asarray(frame)
                frame = cap.retrieve(frame, audioBaseIndex)
                for i in range(len(frame[1][0])):
                    inputAudio.append(frame[1][0][i])
            else:
                break

        inputAudio = np.asarray(inputAudio)
        print("Number of samples: ", len(inputAudio))
        samplingRate = int(cap.get(cv.CAP_PROP_AUDIO_SAMPLES_PER_SECOND))
        return samplingRate, inputAudio


    def readAudioMicrophone(self):
        cap = cv.VideoCapture()

        params = [cv.CAP_PROP_AUDIO_STREAM, 0, cv.CAP_PROP_VIDEO_STREAM, -1]
        params = np.asarray(params)

        cap.open(0, cv.CAP_ANY, params)
        if cap.isOpened() == False:
            print("Error: Can't open microphone")
            print("Error: problems with audio reading, check input arguments")
            exit()
        audioBaseIndex = int(cap.get(cv.CAP_PROP_AUDIO_BASE_INDEX))
        numberOfChannels = int(cap.get(cv.CAP_PROP_AUDIO_TOTAL_CHANNELS))

        print("CAP_PROP_AUDIO_DATA_DEPTH: ", str((int(cap.get(cv.CAP_PROP_AUDIO_DATA_DEPTH)))))
        print("CAP_PROP_AUDIO_SAMPLES_PER_SECOND: ", cap.get(cv.CAP_PROP_AUDIO_SAMPLES_PER_SECOND))
        print("CAP_PROP_AUDIO_TOTAL_CHANNELS: ", numberOfChannels)
        print("CAP_PROP_AUDIO_TOTAL_STREAMS: ", cap.get(cv.CAP_PROP_AUDIO_TOTAL_STREAMS))

        cvTickFreq = cv.getTickFrequency()
        sysTimeCurr = cv.getTickCount()
        sysTimePrev = sysTimeCurr

        frame = []
        frame = np.asarray(frame)
        inputAudio = []

        while ((sysTimeCurr - sysTimePrev) / cvTickFreq < self.microTime):
            if (cap.grab()):
                frame = []
                frame = np.asarray(frame)
                frame = cap.retrieve(frame, audioBaseIndex)
                for i in range(len(frame[1][0])):
                    inputAudio.append(frame[1][0][i])
                sysTimeCurr = cv.getTickCount()
            else:
                print("Error: Grab error")
                break

        inputAudio = np.asarray(inputAudio)
        print("Number of samples: ", len(inputAudio))
        samplingRate = int(cap.get(cv.CAP_PROP_AUDIO_SAMPLES_PER_SECOND))

        return samplingRate, inputAudio


    def drawAmplitude(self, inputAudio):
        color = (247, 111, 87)
        thickness = 5
        frameVectorRows = 500
        middle = frameVectorRows // 2

        # usually the input data is too big, so it is necessary
        # to reduce size using interpolation of data
        frameVectorCols = 40000
        if len(inputAudio) < frameVectorCols:
            frameVectorCols = len(inputAudio)

        img = np.zeros((frameVectorRows, frameVectorCols, 3), np.uint8)
        img += 255  # white background

        audio = np.array(0)
        audio = cv.resize(inputAudio, (1, frameVectorCols), interpolation=cv.INTER_LINEAR)
        reshapeAudio = np.reshape(audio, (-1))

        # normalization data by maximum element
        minCv, maxCv, _, _ = cv.minMaxLoc(reshapeAudio)
        maxElem = int(max(abs(minCv), abs(maxCv)))

        # if all data values are zero (silence)
        if maxElem == 0:
            maxElem = 1
        for i in range(len(reshapeAudio)):
            reshapeAudio[i] = middle - reshapeAudio[i] * middle // maxElem

        for i in range(1, frameVectorCols, 1):
            cv.line(img, (i - 1, int(reshapeAudio[i - 1])), (i, int(reshapeAudio[i])), color, thickness)

        img = cv.resize(img, (900, 400), interpolation=cv.INTER_AREA)
        return img


    def drawAmplitudeScale(self, inputImg, inputAudio, samplingRate, xmin=None, xmax=None):
        # function of layout drawing for graph of volume amplitudes
        # x axis for time
        # y axis for amplitudes

        # parameters for the new image size
        preCol = 100
        aftCol = 100
        preLine = 40
        aftLine = 50

        frameVectorRows = inputImg.shape[0]
        frameVectorCols = inputImg.shape[1]

        totalRows = preLine + frameVectorRows + aftLine
        totalCols = preCol + frameVectorCols + aftCol

        imgTotal = np.zeros((totalRows, totalCols, 3), np.uint8)
        imgTotal += 255  # white background
        imgTotal[preLine: preLine + frameVectorRows, preCol: preCol + frameVectorCols] = inputImg

        # calculating values on x axis
        if xmin is None:
            xmin = 0
        if xmax is None:
            xmax = len(inputAudio) / samplingRate

        if xmax > self.xmarkup:
            xList = np.linspace(xmin, xmax, self.xmarkup).astype(int)
        else:
            # this case is used to display a dynamic update
            tmp = np.arange(xmin, xmax, 1).astype(int) + 1
            xList = np.concatenate((np.zeros(self.xmarkup - len(tmp)), tmp[:]), axis=None)

        # calculating values on y axis
        ymin = np.min(inputAudio)
        ymax = np.max(inputAudio)
        yList = np.linspace(ymin, ymax, self.ymarkup)

        # parameters for layout drawing
        textThickness = 1
        gridThickness = 1
        gridColor = (0, 0, 0)
        textColor = (0, 0, 0)
        font = cv.FONT_HERSHEY_SIMPLEX
        fontScale = 0.5

        # horizontal axis under the graph
        cv.line(imgTotal, (preCol, totalRows - aftLine),
                (preCol + frameVectorCols, totalRows - aftLine),
                gridColor, gridThickness)
        # vertical axis for amplitude
        cv.line(imgTotal, (preCol, preLine), (preCol, preLine + frameVectorRows),
                gridColor, gridThickness)

        # parameters for layout calculation
        serifSize = 10
        indentDownX = serifSize * 2
        indentDownY = serifSize // 2
        indentLeftX = serifSize
        indentLeftY = 2 * preCol // 3

        # drawing layout for x axis
        numX = frameVectorCols // (self.xmarkup - 1)
        for i in range(len(xList)):
            a1 = preCol + i * numX
            a2 = frameVectorRows + preLine
            b1 = a1
            b2 = a2 + serifSize
            if self.enableGrid is True:
                d1 = a1
                d2 = preLine
                cv.line(imgTotal, (a1, a2), (d1, d2), gridColor, gridThickness)
            cv.line(imgTotal, (a1, a2), (b1, b2), gridColor, gridThickness)
            cv.putText(imgTotal, str(int(xList[i])), (b1 - indentLeftX, b2 + indentDownX),
                    font, fontScale, textColor, textThickness)

        # drawing layout for y axis
        numY = frameVectorRows // (self.ymarkup - 1)
        for i in range(len(yList)):
            a1 = preCol
            a2 = totalRows - aftLine - i * numY
            b1 = preCol - serifSize
            b2 = a2
            if self.enableGrid is True:
                d1 = preCol + frameVectorCols
                d2 = a2
                cv.line(imgTotal, (a1, a2), (d1, d2), gridColor, gridThickness)
            cv.line(imgTotal, (a1, a2), (b1, b2), gridColor, gridThickness)
            cv.putText(imgTotal, str(int(yList[i])), (b1 - indentLeftY, b2 + indentDownY),
                    font, fontScale, textColor, textThickness)
        imgTotal = cv.resize(imgTotal, (self.cols, self.rows), interpolation=cv.INTER_AREA)
        return imgTotal


    def STFT(self, inputAudio):
        """
        The Short-time Fourier transform (STFT), is a Fourier-related transform used to determine
        the sinusoidal frequency and phase content of local sections of a signal as it changes over
        time.
        In practice, the procedure for computing STFTs is to divide a longer time signal into
        shorter segments of equal length and then compute the Fourier transform separately on each
        shorter segment. This reveals the Fourier spectrum on each shorter segment. One then usually
        plots the changing spectra as a function of time, known as a spectrogram or waterfall plot.

        https://en.wikipedia.org/wiki/Short-time_Fourier_transform
        """

        time_step = self.windLen - self.overlap
        if time_step <= 0:
            raise ValueError(
                "Invalid STFT parameters: overlap must be smaller than window length"
            )
        stft = []

        if self.windowType == "Hann":
            # https://en.wikipedia.org/wiki/Window_function#Hann_and_Hamming_windows
            Hann_wind = []
            for i in range (1 - self.windLen, self.windLen, 2):
                Hann_wind.append(i * (0.5 + 0.5 * math.cos(math.pi * i / (self.windLen - 1))))
            Hann_wind = np.asarray(Hann_wind)

        elif self.windowType == "Hamming":
            # https://en.wikipedia.org/wiki/Window_function#Hann_and_Hamming_windows
            Hamming_wind = []
            for i in range (1 - self.windLen, self.windLen, 2):
                Hamming_wind.append(i * (0.53836 - 0.46164 * (math.cos(2 * math.pi * i / (self.windLen - 1)))))
            Hamming_wind = np.asarray(Hamming_wind)

        for index in np.arange(0, len(inputAudio), time_step).astype(int):

            section = inputAudio[index:index + self.windLen]
            zeroArray = np.zeros(self.windLen - len(section))
            section = np.concatenate((section, zeroArray), axis=None)

            if self.windowType == "Hann":
                section *= Hann_wind
            elif self.windowType == "Hamming":
                section *= Hamming_wind

            dst = np.empty(0)
            dst = cv.dft(section, dst, flags=cv.DFT_COMPLEX_OUTPUT)
            reshape_dst = np.reshape(dst, (-1))
            # we need only the first part of the spectrum, the second part is symmetrical
            complexArr = np.zeros(len(dst) // 4, dtype=complex)
            for i in range(len(dst) // 4):
                complexArr[i] = complex(reshape_dst[2 * i], reshape_dst[2 * i + 1])
            stft.append(np.abs(complexArr))

        stft = np.array(stft).transpose()
        # convert elements to the decibel scale
        np.log10(stft, out=stft, where=(stft != 0.))
        return 10 * stft


    def drawSpectrogram(self, stft):

        frameVectorRows = stft.shape[0]
        frameVectorCols = stft.shape[1]

        # Normalization of image values from 0 to 255 to get more contrast image
        # and this normalization will be taken into account in the scale drawing
        colormapImageRows = 255

        imgSpec = np.zeros((frameVectorRows, frameVectorCols, 3), np.uint8)
        stftMat = np.zeros((frameVectorRows, frameVectorCols), np.float64)
        cv.normalize(stft, stftMat, 1.0, 0.0, cv.NORM_INF)

        for i in range(frameVectorRows):
            for j in range(frameVectorCols):
                imgSpec[frameVectorRows - i - 1, j] = int(stftMat[i][j] * colormapImageRows)

        imgSpec = cv.applyColorMap(imgSpec, cv.COLORMAP_INFERNO)
        imgSpec = cv.resize(imgSpec, (900, 400), interpolation=cv.INTER_LINEAR)
        return imgSpec


    def drawSpectrogramColorbar(self, inputImg, inputAudio, samplingRate, stft, xmin=None, xmax=None):
        # function of layout drawing for the three-dimensional graph of the spectrogram
        # x axis for time
        # y axis for frequencies
        # z axis for magnitudes of frequencies shown by color scale

        # parameters for the new image size
        preCol = 100
        aftCol = 100
        preLine = 40
        aftLine = 50
        colColor = 20
        ind_col = 20

        frameVectorRows = inputImg.shape[0]
        frameVectorCols = inputImg.shape[1]

        totalRows = preLine + frameVectorRows + aftLine
        totalCols = preCol + frameVectorCols + aftCol + colColor

        imgTotal = np.zeros((totalRows, totalCols, 3), np.uint8)
        imgTotal += 255  # white background
        imgTotal[preLine: preLine + frameVectorRows, preCol: preCol + frameVectorCols] = inputImg

        # colorbar image due to drawSpectrogram(..) picture has been normalised from 255 to 0,
        # so here colorbar has values from 255 to 0
        colorArrSize = 256
        imgColorBar = np.zeros((colorArrSize, colColor, 1), np.uint8)

        for i in range(colorArrSize):
            imgColorBar[i] += colorArrSize - 1 - i

        imgColorBar = cv.applyColorMap(imgColorBar, cv.COLORMAP_INFERNO)
        imgColorBar = cv.resize(imgColorBar, (colColor, frameVectorRows), interpolation=cv.INTER_AREA)  #

        imgTotal[preLine: preLine + frameVectorRows,
        preCol + frameVectorCols + ind_col:
        preCol + frameVectorCols + ind_col + colColor] = imgColorBar

        # calculating values on x axis
        if xmin is None:
            xmin = 0
        if xmax is None:
            xmax = len(inputAudio) / samplingRate
        if xmax > self.xmarkup:
            xList = np.linspace(xmin, xmax, self.xmarkup).astype(int)
        else:
            # this case is used to display a dynamic update
            tmpXList = np.arange(xmin, xmax, 1).astype(int) + 1
            xList = np.concatenate((np.zeros(self.xmarkup - len(tmpXList)), tmpXList[:]), axis=None)

        # calculating values on y axis
        # according to the Nyquist sampling theorem,
        # signal should posses frequencies equal to half of sampling rate
        ymin = 0
        ymax = int(samplingRate / 2.)
        yList = np.linspace(ymin, ymax, self.ymarkup).astype(int)

        # calculating values on z axis
        zList = np.linspace(np.min(stft), np.max(stft), self.zmarkup)

        # parameters for layout drawing
        textThickness = 1
        textColor = (0, 0, 0)
        gridThickness = 1
        gridColor = (0, 0, 0)
        font = cv.FONT_HERSHEY_SIMPLEX
        fontScale = 0.5

        serifSize = 10
        indentDownX = serifSize * 2
        indentDownY = serifSize // 2
        indentLeftX = serifSize
        indentLeftY = 2 * preCol // 3

        # horizontal axis
        cv.line(imgTotal, (preCol, totalRows - aftLine), (preCol + frameVectorCols, totalRows - aftLine),
                gridColor, gridThickness)
        # vertical axis
        cv.line(imgTotal, (preCol, preLine), (preCol, preLine + frameVectorRows),
                gridColor, gridThickness)

        # drawing layout for x axis
        numX = frameVectorCols // (self.xmarkup - 1)
        for i in range(len(xList)):
            a1 = preCol + i * numX
            a2 = frameVectorRows + preLine
            b1 = a1
            b2 = a2 + serifSize
            cv.line(imgTotal, (a1, a2), (b1, b2), gridColor, gridThickness)
            cv.putText(imgTotal, str(int(xList[i])), (b1 - indentLeftX, b2 + indentDownX),
                    font, fontScale, textColor, textThickness)

        # drawing layout for y axis
        numY = frameVectorRows // (self.ymarkup - 1)
        for i in range(len(yList)):
            a1 = preCol
            a2 = totalRows - aftLine - i * numY
            b1 = preCol - serifSize
            b2 = a2
            cv.line(imgTotal, (a1, a2), (b1, b2), gridColor, gridThickness)
            cv.putText(imgTotal, str(int(yList[i])), (b1 - indentLeftY, b2 + indentDownY),
                    font, fontScale, textColor, textThickness)

        # drawing layout for z axis
        numZ = frameVectorRows // (self.zmarkup - 1)
        for i in range(len(zList)):
            a1 = preCol + frameVectorCols + ind_col + colColor
            a2 = totalRows - aftLine - i * numZ
            b1 = a1 + serifSize
            b2 = a2
            cv.line(imgTotal, (a1, a2), (b1, b2), gridColor, gridThickness)
            cv.putText(imgTotal, str(int(zList[i])), (b1 + 10, b2 + indentDownY),
                    font, fontScale, textColor, textThickness)
        imgTotal = cv.resize(imgTotal, (self.cols, self.rows), interpolation=cv.INTER_AREA)
        return imgTotal


    def concatenateImages(self, img1, img2):
        # first image will be under the second image
        totalRows = img1.shape[0] + img2.shape[0]
        totalCols = max(img1.shape[1], img2.shape[1])

        # if images columns do not match, the difference is filled in white
        imgTotal = np.zeros((totalRows, totalCols, 3), np.uint8)
        imgTotal += 255

        imgTotal[:img1.shape[0], :img1.shape[1]] = img1
        imgTotal[img2.shape[0]:, :img2.shape[1]] = img2

        return imgTotal


    def dynamicFile(self, file):
        cap = cv.VideoCapture(file)
        params = [cv.CAP_PROP_AUDIO_STREAM, self.audioStream,
                cv.CAP_PROP_VIDEO_STREAM, -1,
                cv.CAP_PROP_AUDIO_DATA_DEPTH, cv.CV_16S]
        params = np.asarray(params)

        cap.open(file, cv.CAP_ANY, params)
        if cap.isOpened() == False:
            print("ERROR! Can't to open file")
            return

        audioBaseIndex = int(cap.get(cv.CAP_PROP_AUDIO_BASE_INDEX))
        numberOfChannels = int(cap.get(cv.CAP_PROP_AUDIO_TOTAL_CHANNELS))
        samplingRate = int(cap.get(cv.CAP_PROP_AUDIO_SAMPLES_PER_SECOND))

        print("CAP_PROP_AUDIO_DATA_DEPTH: ", str((int(cap.get(cv.CAP_PROP_AUDIO_DATA_DEPTH)))))
        print("CAP_PROP_AUDIO_SAMPLES_PER_SECOND: ", cap.get(cv.CAP_PROP_AUDIO_SAMPLES_PER_SECOND))
        print("CAP_PROP_AUDIO_TOTAL_CHANNELS: ", numberOfChannels)
        print("CAP_PROP_AUDIO_TOTAL_STREAMS: ", cap.get(cv.CAP_PROP_AUDIO_TOTAL_STREAMS))

        step = int(self.updateTime * samplingRate)
        frameSize = int(self.frameSizeTime * samplingRate)
        # since the dimensional grid is counted in integer seconds,
        # if duration of audio frame is less than xmarkup, to avoid an incorrect display,
        # xmarkup will be taken equal to duration
        if self.frameSizeTime <= self.xmarkup:
            self.xmarkup = self.frameSizeTime

        buffer = []
        section = np.zeros(frameSize, dtype=np.int16)
        currentSamples = 0

        while (1):
            if (cap.grab()):
                frame = []
                frame = np.asarray(frame)
                frame = cap.retrieve(frame, audioBaseIndex)

                for i in range(len(frame[1][0])):
                    buffer.append(frame[1][0][i])

                buffer_size = len(buffer)
                if (buffer_size >= step):

                    section = list(section)
                    currentSamples += step

                    del section[0:step]
                    section.extend(buffer[0:step])
                    del buffer[0:step]

                    section = np.asarray(section)

                    if currentSamples < frameSize:
                        xmin = 0
                        xmax = (currentSamples) / samplingRate
                    else:
                        xmin = (currentSamples - frameSize) / samplingRate + 1
                        xmax = (currentSamples) / samplingRate

                    if self.graph == "ampl":
                        imgAmplitude = self.drawAmplitude(section)
                        imgAmplitude = self.drawAmplitudeScale(imgAmplitude, section, samplingRate, xmin, xmax)
                        cv.imshow("Display amplitude graph", imgAmplitude)
                        cv.waitKey(self.waitTime)

                    elif self.graph == "spec":
                        stft = self.STFT(section)
                        imgSpec = self.drawSpectrogram(stft)
                        imgSpec = self.drawSpectrogramColorbar(imgSpec, section, samplingRate, stft, xmin, xmax)
                        cv.imshow("Display spectrogram", imgSpec)
                        cv.waitKey(self.waitTime)

                    elif self.graph == "ampl_and_spec":

                        imgAmplitude = self.drawAmplitude(section)
                        stft = self.STFT(section)
                        imgSpec = self.drawSpectrogram(stft)

                        imgAmplitude = self.drawAmplitudeScale(imgAmplitude, section, samplingRate, xmin, xmax)
                        imgSpec = self.drawSpectrogramColorbar(imgSpec, section, samplingRate, stft, xmin, xmax)

                        imgTotal = self.concatenateImages(imgAmplitude, imgSpec)
                        cv.imshow("Display amplitude graph and spectrogram", imgTotal)
                        cv.waitKey(self.waitTime)
            else:
                break


    def dynamicMicrophone(self):
        cap = cv.VideoCapture()
        params = [cv.CAP_PROP_AUDIO_STREAM, 0, cv.CAP_PROP_VIDEO_STREAM, -1]
        params = np.asarray(params)

        cap.open(0, cv.CAP_ANY, params)
        if cap.isOpened() == False:
            print("ERROR! Can't to open file")
            return
        audioBaseIndex = int(cap.get(cv.CAP_PROP_AUDIO_BASE_INDEX))
        numberOfChannels = int(cap.get(cv.CAP_PROP_AUDIO_TOTAL_CHANNELS))

        print("CAP_PROP_AUDIO_DATA_DEPTH: ", str((int(cap.get(cv.CAP_PROP_AUDIO_DATA_DEPTH)))))
        print("CAP_PROP_AUDIO_SAMPLES_PER_SECOND: ", cap.get(cv.CAP_PROP_AUDIO_SAMPLES_PER_SECOND))
        print("CAP_PROP_AUDIO_TOTAL_CHANNELS: ", numberOfChannels)
        print("CAP_PROP_AUDIO_TOTAL_STREAMS: ", cap.get(cv.CAP_PROP_AUDIO_TOTAL_STREAMS))

        frame = []
        frame = np.asarray(frame)
        samplingRate = int(cap.get(cv.CAP_PROP_AUDIO_SAMPLES_PER_SECOND))

        step = int(self.updateTime * samplingRate)
        frameSize = int(self.frameSizeTime * samplingRate)
        self.xmarkup = self.frameSizeTime

        currentSamples = 0

        buffer = []
        section = np.zeros(frameSize, dtype=np.int16)

        cvTickFreq = cv.getTickFrequency()
        sysTimeCurr = cv.getTickCount()
        sysTimePrev = sysTimeCurr
        self.waitTime = self.updateTime * 1000
        while ((sysTimeCurr - sysTimePrev) / cvTickFreq < self.microTime):
            if (cap.grab()):
                frame = []
                frame = np.asarray(frame)
                frame = cap.retrieve(frame, audioBaseIndex)

                for i in range(len(frame[1][0])):
                    buffer.append(frame[1][0][i])

                sysTimeCurr = cv.getTickCount()
                buffer_size = len(buffer)
                if (buffer_size >= step):

                    section = list(section)
                    currentSamples += step

                    del section[0:step]
                    section.extend(buffer[0:step])
                    del buffer[0:step]

                    section = np.asarray(section)

                    if currentSamples < frameSize:
                        xmin = 0
                        xmax = (currentSamples) / samplingRate
                    else:
                        xmin = (currentSamples - frameSize) / samplingRate + 1
                        xmax = (currentSamples) / samplingRate

                    if self.graph == "ampl":
                        imgAmplitude = self.drawAmplitude(section)
                        imgAmplitude = self.drawAmplitudeScale(imgAmplitude, section, samplingRate, xmin, xmax)
                        cv.imshow("Display amplitude graph", imgAmplitude)
                        cv.waitKey(self.waitTime)

                    elif self.graph == "spec":
                        stft = self.STFT(section)
                        imgSpec = self.drawSpectrogram(stft)
                        imgSpec = self.drawSpectrogramColorbar(imgSpec, section, samplingRate, stft, xmin, xmax)
                        cv.imshow("Display spectrogram", imgSpec)
                        cv.waitKey(self.waitTime)

                    elif self.graph == "ampl_and_spec":
                        imgAmplitude = self.drawAmplitude(section)
                        stft = self.STFT(section)
                        imgSpec = self.drawSpectrogram(stft)

                        imgAmplitude = self.drawAmplitudeScale(imgAmplitude, section, samplingRate, xmin, xmax)
                        imgSpec = self.drawSpectrogramColorbar(imgSpec, section, samplingRate, stft, xmin, xmax)

                        imgTotal = self.concatenateImages(imgAmplitude, imgSpec)
                        cv.imshow("Display amplitude graph and spectrogram", imgTotal)
                        cv.waitKey(self.waitTime)
            else:
                break


    def initAndCheckArgs(self, args):
        if args.inputType != "file" and args.inputType != "microphone":
            print("Error: ", args.inputType, " input method doesnt exist")
            return False
        if args.draw != "static" and args.draw != "dynamic":
            print("Error: ", args.draw, " draw type doesnt exist")
            return False
        if args.graph != "ampl" and args.graph != "spec" and args.graph != "ampl_and_spec":
            print("Error: ", args.graph, " type of graph doesnt exist")
            return False
        if args.windowType != "Rect" and args.windowType != "Hann" and args.windowType != "Hamming":
            print("Error: ", args.windowType, " type of window doesnt exist")
            return False
        if args.windLen <= 0:
            print("Error: windLen = ", args.windLen, " - incorrect value. Must be > 0")
            return False
        if args.overlap <= 0:
            print("Error: overlap = ", args.overlap, " - incorrect value. Must be > 0")
            return False
        if args.rows <= 0:
            print("Error: rows = ", args.rows, " - incorrect value. Must be > 0")
            return False
        if args.cols <= 0:
            print("Error: cols = ", args.cols, " - incorrect value. Must be > 0")
            return False
        if args.xmarkup < 2:
            print("Error: xmarkup = ", args.xmarkup, " - incorrect value. Must be >= 2")
            return False
        if args.ymarkup < 2:
            print("Error: ymarkup = ", args.ymarkup, " - incorrect value. Must be >= 2")
            return False
        if args.zmarkup < 2:
            print("Error: zmarkup = ", args.zmarkup, " - incorrect value. Must be >= 2")
            return False
        if args.microTime <= 0:
            print("Error: microTime = ", args.microTime, " - incorrect value. Must be > 0")
            return False
        if args.frameSizeTime <= 0:
            print("Error: frameSizeTime = ", args.frameSizeTime, " - incorrect value. Must be > 0")
            return False
        if args.updateTime <= 0:
            print("Error: updateTime = ", args.updateTime, " - incorrect value. Must be > 0")
            return False
        if args.waitTime < 0:
            print("Error: waitTime = ", args.waitTime, " - incorrect value. Must be >= 0")
            return False
        return True


if __name__ == "__main__":

    parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter,
                                     description='''this sample draws a volume graph and/or spectrogram of audio/video files and microphone\nDefault usage: ./Spectrogram.exe''')

    parser.add_argument("-i", "--inputType", dest="inputType", type=str, default="file", help="file or microphone")
    parser.add_argument("-d", "--draw", dest="draw", type=str, default="static",
                        help="type of drawing: static - for plotting graph(s) across the entire input audio; dynamic - for plotting graph(s) in a time-updating window")
    parser.add_argument("-g", "--graph", dest="graph", type=str, default="ampl_and_spec",
                        help="type of graph: amplitude graph or/and spectrogram. Please use tags below : ampl - draw the amplitude graph; spec - draw the spectrogram; ampl_and_spec - draw the amplitude graph and spectrogram on one image under each other")

    parser.add_argument("-a", "--audio", dest="audio", type=str, default='Megamind.avi',
                        help="name and path to file")
    parser.add_argument("-s", "--audioStream", dest="audioStream", type=int, default=1,
                        help=" CAP_PROP_AUDIO_STREAM value")

    parser.add_argument("-t", '--windowType', dest="windowType", type=str, default="Rect",
                        help="type of window for STFT. Please use tags below : Rect/Hann/Hamming")
    parser.add_argument("-l", '--windLen', dest="windLen", type=int, default=256, help="size of window for STFT")
    parser.add_argument("-o", '--overlap', dest="overlap", type=int, default=128, help="overlap of windows for STFT")

    parser.add_argument("-gd", '--grid', dest="enableGrid", type=bool, default=False, help="grid on amplitude graph(on/off)")

    parser.add_argument("-r", '--rows', dest="rows", type=int, default=400, help="rows of output image")
    parser.add_argument("-c", '--cols', dest="cols", type=int, default=900, help="cols of output image")

    parser.add_argument("-x", '--xmarkup', dest="xmarkup", type=int, default=5,
                        help="number of x axis divisions (time asix)")
    parser.add_argument("-y", '--ymarkup', dest="ymarkup", type=int, default=5,
                        help="number of y axis divisions (frequency or/and amplitude axis)")  # ?
    parser.add_argument("-z", '--zmarkup', dest="zmarkup", type=int, default=5,
                        help="number of z axis divisions (colorbar)")  # ?

    parser.add_argument("-m", '--microTime', dest="microTime", type=int, default=20,
                        help="time of recording audio with microphone in seconds")
    parser.add_argument("-f", '--frameSizeTime', dest="frameSizeTime", type=int, default=5,
                        help="size of sliding window in seconds")
    parser.add_argument("-u", '--updateTime', dest="updateTime", type=int, default=1,
                        help="update time of sliding window in seconds")
    parser.add_argument("-w", '--waitTime', dest="waitTime", type=int, default=10,
                        help="parameter to cv.waitKey() for dynamic update, takes values in milliseconds")

    args = parser.parse_args()

    AudioDrawing(args).Draw()


__all__: list[str] = []

import cv2
import cv2.typing
import typing as _typing


# Classes
class WeChatQRCode:
    # Functions
    def __init__(self, detector_prototxt_path: str = ..., detector_caffe_model_path: str = ..., super_resolution_prototxt_path: str = ..., super_resolution_caffe_model_path: str = ...) -> None: ...

    @_typing.overload
    def detectAndDecode(self, img: cv2.typing.MatLike, points: _typing.Sequence[cv2.typing.MatLike] | None = ...) -> tuple[_typing.Sequence[str], _typing.Sequence[cv2.typing.MatLike]]: ...
    @_typing.overload
    def detectAndDecode(self, img: cv2.UMat, points: _typing.Sequence[cv2.UMat] | None = ...) -> tuple[_typing.Sequence[str], _typing.Sequence[cv2.UMat]]: ...

    def setScaleFactor(self, _scalingFactor: float) -> None: ...

    def getScaleFactor(self) -> float: ...



